#Used for model_selection
calc_simple_likelihood=function(objects, best_call=T)
{
  use_null        = objects$flags$use_null_class
  anocl           = objects$opars$anocl #The number of classes that an individual can be assigned to (maybe NULL)
  vals            = as.numeric(rownames(objects$dat)) # The grid values which the dat2 records the counts for.
  ld_gv           = make_combined_lpdga(objects=objects, vals=vals, use_null=use_null)
  #class proportions (same as population genotype frequencies) (repeated to allow matrix calculations)
 #avgnf           = apply(objects$opars$gnfp$gnf, 1, mean)
  dat2            = rowSums(objects$dat)
  if(use_null){
   #avgnf = c(avgnf, objects$opars$null_prop)
   #avgnf = normalise(avgnf)
   #names(avgnf)[length(avgnf)] = "NULL"
    full_props = c(objects$opars$gnfp$gnf_lambda, objects$opars$null_prop) #Use gnf_lambda here as with no association model just want poplation frequencies.
    props_mat_colnames = c(colnames(objects$opars$clv), "NULL")
  } else      {
    full_props = objects$opars$gnfp$gnf_lambda
    props_mat_colnames = c(colnames(objects$opars$clv))
  }
     # THink the next two (commented) lines are confused, I've put in the maths and hopefully improved lines below.
     #l_lk_gv         = joint_dens_prop*apply(objects$dat, 1, sum)
     #l_lk            = apply(l_lk_gv, 1, add_log2)
  # Likelihood is Product_(individuals) sum_(classes cj) P(X_i | u_cj, s^2_cj, i in cj) P(i in cj)
  # P(X_i | u_cj, s^2_cj, i in cj) = ld_gv[i, cj]
 #class_prop_mat  = matrix(avgnf, nrow=length(vals), ncol=anocl, byrow=T)
  class_prop_mat  = matrix(full_props, nrow=length(vals), ncol=anocl, byrow=T)
  colnames(class_prop_mat)  = props_mat_colnames
  joint_dens_prop = ld_gv + log(class_prop_mat)      #These are log(likelihood(each individual, class)). Likelihood per ind is the sum over classes.
  if( best_call) 
    lpeach_ind      = apply(joint_dens_prop, 1, max) #These are log(likelihood(each individual|best_call)). Total likelihood is the product over individuals.
  if(!best_call) {
    if(objects$flags$DEBUG>0) {cat("ERROR! Should be conditioning on calls, that means using the best posterior call.\n"); browser()}
    lpeach_ind      = apply(joint_dens_prop, 1, add_log2) #These are log(likelihood(each individual       )). Total likelihood is the product over individuals.
  }
  l_lk            = sum(lpeach_ind * dat2)
  return(l_lk)
}

##Calculating posteriors on CNV calls:
update_lpdga=function(objects = NULL, null_prop=objects$npars$null_prop)
{
  use_null  = objects$flags$use_null_class
  noco      = objects$opars$noco
  nocl      = objects$opars$nocl  #The number of possible genotypes that an individual can be assigned to (so - not including NULL)
  anocl     = objects$opars$anocl #The number of classes that an individual can be assigned to (maybe NULL)
 #if(objects$flags$DEBUG>1.5 & objects$flags$iter>0 & abs(objects$likelihood$lposterior)<1E-3) {cat("Warning, small likelihood.\n"); browser();}
  if(is.null(objects)) {cat("In update_lpdga. Null objects passed.\n"); browser(); return(1);}
  vals = as.numeric(rownames(objects$dat)) #These are the centres of the bins that the data was gridded to.
  for(coll in 1:noco)
  {
    #tmp           = c(objects$npars$smc[coll], (objects$npars$cld[coll,]))
    #class_means   = cumsum(tmp)      
     class_means   = objects$npars$hkclm[coll,]
     #for(cl in 1:anocl)
     #{
     #  #The values for the bins.
     #  if(use_null & cl == anocl) {
     #    l_dnsty = dunif(x=vals, min=min(vals), max=max(vals), log=T)
     #  } else                     {
     #    if(objects$flags$signal_model=="normal")
     #      l_dnsty = dnorm(x  =vals      , mean=class_means[cl], sd=sqrt(objects$npars$hkclv[coll,cl]), log=T)
     #    if(objects$flags$signal_model=="t8") {
     #      l_dnsty = my_dt(dat=vals, df=8, mean=class_means[cl], sd=sqrt(objects$npars$hkclv[coll,cl]), log=T)
     #    }
     #  }
     #  objects$ninferred$lpdga[,cl,coll]  = l_dnsty
     #} 
     if(objects$flags$signal_model=="normal")
       objects$ninferred$lpdga[,1:nocl,coll] = mapply(dnorm, mean=class_means, sd=sqrt(objects$npars$hkclv[coll,]), MoreArgs=list("x"=vals, log=T))
     if(objects$flags$signal_model=="t8")
       objects$ninferred$lpdga[,1:nocl,coll] = mapply(my_dt, mean=class_means, sd=sqrt(objects$npars$hkclv[coll,]), MoreArgs=list("dat"=vals, "log"=T, "df"=8))
     if(objects$flags$signal_model=="t")
       objects$ninferred$lpdga[,1:nocl,coll] = mapply(my_dt, mean=class_means, sd=sqrt(objects$npars$hkclv[coll,]), MoreArgs=list("dat"=vals, "log"=T, "df"=objects$flags$deg_free))
     if(use_null) objects$ninferred$lpdga[,anocl,coll] = dunif(x=vals, min=min(vals), max=max(vals), log=T)

     if(0) {cat("  Check new lpdga is working\n"); browser();}


     if(is.na(range(objects$ninferred$lpdga)[1])) {cat("Error - Invalid densities:", range(objects$ninferred$lpdga)[1], "\n"); browser();}
     if(0)  #Probably not needed code:
     {
       gnf_lambda = objects$npars$gnfp$gnf_lambda #Depending on the model I may wish to distinguish between cases and controls!
       aprobs = make_aprobs(gnf=NULL, use_null=use_null, null_prop=null_prop, null_freq=ob$gnfp$nullf, use_null_freq=F, noco=noco, gnf_lambda=gnf_lambda)
       for(cl in 1:anocl)
         objects$ninferred$lpagd[,cl,coll]  = objects$ninferred$lpdga[,cl,coll] + log(aprobs[cl,coll]) ##(!!) This looks wrong as it doesn't condition on case/control
  
       lnorm_vals = apply(objects$ninferred$lpagd[,,coll,drop=F], 1, add_log2)
       #The next line gives the probability of (this) assigment in this collection given the data
       objects$ninferred$lpagd[,,coll]  = objects$ninferred$lpagd[,,coll,drop=F] - lnorm_vals
     }
  }
  #objects$ninferred$pagd = exp(objects$ninferred$lpagd)
 #	if(is.na(max(range(objects$ninferred$lpagd)[1], range(objects$ninferred$lpdga)[1]))) {cat("Problems with cnv calls.\n"); browser();}
  if(sum(objects$ninferred$lpdga==-1) == length(objects$ninferred$lpdga)) {cat("In update_lpdga. Problems or degenerate model fit.\n"); browser();}
  return(objects);
}
##Calculating posteriors on CNV calls:


#Shouldn't I do inference on all the affect sizes at once?? Could have a prior on joint effect sizes which 
#says that normally one 1 is significant, that 58C is set at 1 and that auto-immune correlate?
#Calculate the probability of the phenotypes given the new genotype calls (or probabilities?)
#To use this function to find better betas: o=calc_phen_g_gen(objects=f, direct_betas = c(beta0, beta1)) 
calc_phen_g_gen=function(objects, hard_calls=F, new=T, direct_betas = NULL, precision=1e-5)
{
   nocl  = objects$opars$nocl
   noco  = objects$opars$noco
   if(!is.null(direct_betas)) betas = direct_betas
   if( is.null(direct_betas)) {
     if(new) betas = objects$npars$betas    else betas = objects$opars$betas
   }
   #gnf stands for 'genotype frequencies' lambda is the current best guess at the population frequencies in CONTROLS
  #if(new)     gnf_controls = normalise(objects$npars$gnfp$gnf_lambda, sum=1) else gnf_controls   = normalise(objects$opars$gnfp$gnf_lambda, sum=1)
  #gnf_controls = make_control_freqs(objects=objects, pop_freqs =objects$npars$gnfp$gnf_lambda, new=new)
  #gnf_cases    = make_case_freqs(   objects=objects, pop_freqs =objects$npars$gnfp$gnf_lambda, new=new)
  #if( new) stored_gnf_cases    = objects$npars$gnfp$gnf_cases    else stored_gnf_cases    = objects$opars$gnfp$gnf_cases
  #if( new) stored_gnf_controls = objects$npars$gnfp$gnf_controls else stored_gnf_controls = objects$opars$gnfp$gnf_controls
  #check_gnfcs  = sum(abs(normalise(gnf_cases, sum=(1-objects$opars$null_prop))-stored_gnf_cases))<precision
  #check_gnfcs  = check_gnfcs & sum(abs(normalise(gnf_controls, sum=(1-objects$opars$null_prop))-stored_gnf_controls))<precision
  #if(objects$flags$DEBUG>0.5 & !check_gnfcs)  {cat("In calc_phen_g_gen. gnf_cases and/or controls may be wrong.\n"); browser()}

  gnf_lambda   = objects$npars$gnfp$gnf_lambda
  #exp_term                = exp(betas[1] + betas[2]*(1:nocl))   
  exp_term                = exp(betas[1] + betas[2]*(0:(nocl-1)))   
  prob_CASES_given_dat    = exp_term/(1+exp_term) #comes from rearranging: log(p/1-p) = B0 + B1*Z
  prob_controls_given_dat =        1/(1+exp_term)
  lprob_CASES_given_dat    = log(   prob_CASES_given_dat) 
  lprob_controls_given_dat = log(prob_controls_given_dat)
  if(objects$flags$use_null_class) {
    #This is an attempt to set: P(Phenotype | Genotype==NULL) = 1 (Conceivably it should be more like P(Phen) (unconditional on genotype?)
    weighted_sum            = add_log2(lprob_controls_given_dat + log(normalise(objects$opars$gnfp$gnf_lambda, sum=1)))
    lprob_CASES_given_dat    = c(lprob_CASES_given_dat   , log(1-exp(weighted_sum)))
    lprob_controls_given_dat = c(lprob_controls_given_dat,           weighted_sum)
  }
  # exp(add_log(prob_CASES_given_dat, prob_controls_given_dat))
    
  
  caseNames    = names(which(objects$assoc$case_colls==1))
  controlNames = names(which(objects$assoc$case_colls==0))
  if(hard_calls)
  {
    # P(Phi | Z, beta, lambda)
    total_prob  = 0; #Working with log-probability here
    grid_hcalls = make_grid_calls(objects=objects, direct_lpdga=NULL, new=T)
    for(coll in controlNames) {
      total_prob = total_prob + sum(objects$dat[,coll]*lprob_controls_given_dat[grid_hcalls[,coll]])
      if(is.na(sum(total_prob))) {cat("In calc_phen_g_gen. NA probabilities.\n"); browser()}
   }
   for(coll in caseNames) {
     total_prob = total_prob + sum(objects$dat[,coll]*lprob_CASES_given_dat[   grid_hcalls[,coll]])
     if(is.na(sum(total_prob))) {cat("In calc_phen_g_gen. NA probabilities.\n"); browser()}
   }
   objects$assoc$nlkld = sum(total_prob)
  }
  if(!hard_calls)
  {
    # Want: P(Phi |beta, lambda)
    # P(Phi = 1) = sum_k P(Phi=1 | Z=k)P(Z=k)
    #            = sum_k P(Case | genotypes) * POP_FREQS(k)
     lp_case      = log(sum(   prob_CASES_given_dat*gnf_lambda)) # = log(P(Phi=1)) (stands for log prob case)
     lp_control   = log(sum(prob_controls_given_dat*gnf_lambda)) # = log(P(Phi=0)) (stands for log prob control)
     nlkhd        = sum(lp_control*sum(objects$dat[,controlNames]) + lp_case*sum(objects$dat[,caseNames]))
     objects$assoc$nlkld = nlkhd

     if(0 & objects$flags$DEBUG > 1 & betas[2]>1e-9) {cat("Examining what happens as betas change.\n"); browser();}
    #lpcolls    = rep(NA, noco)                                # = log(P(whole collection))
    #for(coll in which(objects$assoc$case_colls==1)) {
    #  lpcolls[coll] = lp_case    * sum(objects$dat[,coll])
    #}
    #for(coll in which(objects$assoc$case_colls==0)) {
    #  lpcolls[coll] = lp_control * sum(objects$dat[,coll])
    #}
    #objects$assoc$nlkld = sum(lpcolls)
  }
  if(objects$flags$DEBUG>1.5 & objects$assoc$nlkld>0)         {cat("Warning - Confused as to how log(P(phen|gen))>0.\n"     ); browser();}
  if(objects$flags$DEBUG>0.5 & abs(objects$assoc$nlkld)==Inf) {cat("Warning - Association likelihood has gone to infinity.\n"); browser();}
  return(objects)
}
#Calculate the probability of the phenotypes given the new genotype calls (or probabilities?)

calc_intens_lkld=function(dat, lpdga=NULL, log=T, best_call = T, test=F, objects=NULL, new=T, colls_separate=F, precision=1E-5)
{
  noco         = objects$opars$noco
  llikelihood  = rep(NA, noco)
  names(llikelihood) = colnames(dat)
  likelihood   = llikelihood
  n_inds       = sum(dat)
  if(is.null(lpdga)) {
    if( new) lpdga = objects$ninferred$lpdga
    if(!new) lpdga = objects$oinferred$lpdga
   #if( new) lpagd = objects$ninferred$lpagd else lpagd = objects$oinferred$lpagd
    if( new & best_call)  pagd = objects$ninferred$pagd  else  pagd = objects$oinferred$pagd
  } else {
   #lpagd = make_lpagd(ob=objects, direct_lpdga=lpdga, new=new, combined=F); 
    lpagd = NA
    if(best_call) pagd = make_pagd( ob=objects, direct_lpdga=lpdga, new=new, combined=F); 
    if(objects$flags$DEBUG>1.5) {cat("This is slow. Try not to call this.\n"); browser();}
  }
  for(coll in 1:noco)
  {
    collName = colnames(dat)[coll]
    if(best_call) 
    {
     #asgnmnts = matrix(lpdga[,,coll,drop=F], nrow=dim(lpdga)[1], ncol=dim(lpdga)[2])
      asgnmnts = matrix(pagd[,,coll,drop=F], nrow=dim(lpdga)[1], ncol=dim(lpdga)[2])
      calls  = apply(asgnmnts, 1, which.max)
       #CURRENTLY NOT UPDATED TO NEW lpdga, but probably won't ever use this version anyway. (Also would affect bic etc.)
      #if(!log)  likelihood[coll] = exp(sum(lpdga[cbind((1:n_inds), calls, coll)]))
      #if( log) llikelihood[coll] =     sum(lpdga[cbind((1:n_inds), calls, coll)])
      #Just using lpagd is obviously NONSENSE! That's just the P(Z|X) - I'm definitely after P(X|Z*) - ie. P(X| best calls)
      indices = cbind(1:nrow(dat), calls, coll)
      if(!log)  likelihood[collName] = exp(sum(dat[,collName]*lpdga[indices]))
      if( log) llikelihood[collName] =     sum(dat[,collName]*lpdga[indices])
      if(0 & objects$flags$iter==100 & objects$opars$nocl==3) {cat("nocl==3 and on iteration 100. Likelihood conditional on calls. - Pausing.\n"); browser();}
    }
    if(!best_call) 
    {
      # X_i is intensity of ind i. cj are clusters that ind i might be in. u_cj, s^2_cj are the means/vars of that class. 
      # pi_j is the current estimate of the proportion of people in that class
      # Data is gridded, so actually go through grid points in turn. Calculate this for that grid point, then raise to power of people in that point.
      # Likelihood is Product_(individuals)                  P(X_i | u_cj, s^2_cj)
      # Likelihood is Product_(individuals) sum_(classes cj) P(X_i | u_cj, s^2_cj, i in cj) P(i in cj | beta1, case/control_status)
      # P(X_i | u_cj, s^2_cj, i in cj) = lpdga[i, co, coll]
      # lpdga = array(-1,  c(grid_size, nocl, noco)) #[Stands for log(probability of data GIVEN assignments (to clusters)) ]
      # lpd   : The probability of an individual summed over possible assignments
      if( new) { gnf_lambda  = objects$npars$gnfp$gnf_lambda; gnf_controls  = objects$npars$gnfp$gnf_controls; gnf_cases  = objects$npars$gnfp$gnf_cases }
      if(!new) { gnf_lambda  = objects$opars$gnfp$gnf_lambda; gnf_controls  = objects$opars$gnfp$gnf_controls; gnf_cases  = objects$opars$gnfp$gnf_cases }
      if(objects$assoc$case_colls[collName]==0) 
       coll_freqs = gnf_controls
      if(objects$assoc$case_colls[collName]==1) 
      {
       coll_freqs = make_case_freqs(objects=objects, pop_freqs=gnf_lambda, new=new)
       check_csgnf = sum(abs(coll_freqs - gnf_cases))<precision; if(!check_csgnf) {cat("Error - problem with case frequencies:\n"); browser();}
      }
      if( objects$flags$use_null_class) {
      #class_props  = c(gnf[,coll], objects$npars$null_prop)
       class_props  = c(coll_freqs, objects$npars$null_prop)
       class_props  = normalise(class_props)
      } else                            {
       class_props  = coll_freqs
      }
      rep_props    = matrix(class_props, nrow=nrow(lpdga[,,collName,drop=F]), ncol=ncol(lpdga[,,collName,drop=F]), byrow=T)
      #lpd1         = lpdga[,,coll] + log(rep_props)
      #lpd2         = apply(lpd1, 1, add_log2)
      #llikelihood[coll]  = sum(lpd2 * dat[,coll])
      pd1          = exp(lpdga[,,collName]) * rep_props
      # The next four lines deal with a potential underflow issue when observations are very unlikely.
      if(objects$flags$DEBUG>1.51 & min(pd1==0)) {
        cat("  Warning, 0 values (underflow) in probability of assignment\n")
        if(DEBUG>1.71) browser()
      }
      # The following line is a hack to deal with the issue.
      pd1[pd1==0]  = 1e-200
      pd2          = rowSums(pd1)
      llikelihood[collName]  = sum(log(pd2) * dat[,collName])
      #if(!is.null(objects)) if(objects$flags$iter %in% 4:5) browser();
      if(!is.null(objects)) if(coll==1 & objects$flags$DEBUG == 1.911 & objects$flags$iter %in% c()) {source("look_at_likelihood.R", local=T); browser();}
    }
  }
 #cat("Look at hard calls based likelihood\n"); browser()
  if(is.na(sum(llikelihood)) | abs(sum(llikelihood))==Inf) {cat("Error - Invalid Likelihood:", llikelihood, "\n"); browser();}
  if(                 !log) return( prod(likelihood))
  if( colls_separate & log) return(llikelihood)
  if(!colls_separate & log) return( sum(llikelihood))
}


#x = calc_newprior(npars = bf3$npars, npriors=bf3$npriors, flags=bf3$flags)
calc_newprior=function(npars, npriors, log=T, gnf_theta=1, flags, use_null_freq=F)
{
  nocl = npars$nocl

  smc        = sum( dnorm(npars$smc, mean  =   npriors$prsmc$mean, sd = sqrt(npriors$prsmc$var)              , log=T))
  smc_mean   = dnorm(    npriors$prsmc$mean, npriors$hprsmc$mean$mean         , sqrt(npriors$hprsmc$mean$var), log=T )
  smc_var    = dinvgamma(npriors$prsmc$var , scale = npriors$hprsmc$var$scale , shape = npriors$hprsmc$var$shape)
  lsmc_prval = smc + smc_mean + log(smc_var)

  if(sum(!(npars$cld>=0))>0) {cat("Error, cannot have negative distances between clusters.\n"); browser();}
 #if(sum( (npars$cld==0))>0) {cat("Error, should not have zero distances between clusters.\n"); browser();}
  if(0) {
    cld_mv     = st_pars_to_mean_var(distribution="gamma"       ,  scale=npriors$prcld$scale ,  shape=npriors$prcld$shape, verbose=T)
    vals_to_plot = (0:3000)/3000
    x = dgamma(vals_to_plot, scale =   npriors$prcld$scale, shape = npriors$prcld$shape  )
    pdf(file = "Problems with my prior density on the class distances.pdf")
    par(mfcol=c(1,1))
    main = paste("Gamma density curve with mean = ", round(cld_mv["mean"], digits=4), " and variance = ", round(cld_mv["var"], digits=4), "", sep="")
    xlab = paste("Value (scale = ", round(npriors$prcld$scale, digits=4), " and shape = ", round(npriors$prcld$shape, digits=4), ")", sep="")
    plot(vals_to_plot, x, type="l", xlab=xlab, ylab="density", main=main, xlim=c(0,0.1))
   #dev.copy2eps(file = "Problems with my prior density on the class distances.eps")
    dev.off()
  }
 #cld        = prod(dgamma(npars$cld, scale =   npriors$prcld$scale, shape = npriors$prcld$shape  ))
  cld        = sum( dgamma(npars$cld, scale =   npriors$prcld$scale, shape = npriors$prcld$shape, log=T  ))
  cld_mean   = npriors$prcld$scale   * npriors$prcld$shape
  cld_var    = npriors$prcld$scale^2 * npriors$prcld$shape
  prcld_mean = dgamma(cld_mean, scale = npriors$hprcld$mean$scale, shape = npriors$hprcld$mean$shape, log=T)
  prcld_var  = dgamma(cld_var , scale = npriors$hprcld$var$scale , shape = npriors$hprcld$var$shape , log=T)
  lcld_prval = cld + prcld_mean + prcld_var
  if(flags$DEBUG>2.5) {cat("\tlprior of cld is: cld ", (cld), ", cld_mean ", (prcld_mean), ", cld_var ", (prcld_var), "\n", sep="");}

   base      = sum( dnorm(log(npars$clv[,1]), mean  =   npriors$prlclv_base$mean, sd = sqrt(npriors$prlclv_base$var), log=T)) #moved penultimate bracket to after log=T
   inc1      = t(t(log(npars$clv[,-c(1)]) - log(npars$clv[,1]))/1:(nocl-1))
  #inc       = sum(dgamma(exp(inc1) , scale =   npriors$prlclv_inc$scale, shape = npriors$prlclv_inc$shape  ), log=T)
   inc       = sum(dnorm(exp(inc1) , mean =   npriors$prlclv_inc$mean, sd = sqrt(npriors$prlclv_inc$var)  ), log=T)
  base_mean  = dnorm(    npriors$prlclv_base$mean, npriors$hprclv$base_mean$mean         , sqrt(npriors$hprclv$base_mean$var), log=T)
  base_var   = log(dinvgamma(npriors$prlclv_base$var , scale = npriors$hprclv$base_var$scale , shape = npriors$hprclv$base_var$shape))
  inc_mean   = npriors$prlclv_inc$mean
  inc_var    = npriors$prlclv_inc$var
  princ_mean = dgamma(inc_mean, scale = npriors$hprclv$inc_mean$scale, shape = npriors$hprclv$inc_mean$shape, log=T)
  princ_var  = dgamma(inc_var , scale = npriors$hprclv$inc_var$scale , shape = npriors$hprclv$inc_var$shape , log=T)

  clv_prval  = base + base_mean + base_var + inc + princ_mean + princ_var

  #association_npriors
  lassoc_prval = dnorm(npars$betas[1], mean=npriors$prbeta0$mean, sd=sqrt(npriors$prbeta0$var), log=T) +
                 dnorm(npars$betas[2], mean=npriors$prbeta1$mean, sd=sqrt(npriors$prbeta1$var), log=T)
  if(is.na(lassoc_prval)) {cat("In calc_newprior. Problem with priors on betas.\n"); browser();}

  #this calculates the prior probability of the new (proposed) allele_frequency vector. 
  #alpha  = npars$gnfp$gnf_apar
  nocl   = npars$nocl  
  noco   = npars$noco
  #gtype_counts = matrix(-1, nrow = noco, ncol = nocl)
  #for(coll in 1:noco)
  #{
  #  asgnmnts = matrix(lpagd[,,coll,drop=F], nrow=dim(lpagd)[1], ncol=dim(lpagd)[2])
  #  calls  = apply(asgnmnts, 1, which.max)
  #  for(cl in 1:(nocl)) ##Because multinomials can have 0 successes! I may need to update this throughout program!
  #  {
  #    gtype_counts[coll, (cl)] = sum(calls==(cl))
  #  }
  #  gnf_call_prior = dmultinom(x = (gtype_counts[coll,]-1), prob=npars$gnfp$gnf_lambda, log=T)
  #}  ## NOT NEEDED, the probabilities of individual assignments taken account of in (intensity) LIKELIHOOD calculation

  # (!!) (!!!) There is a problem with the current implementation. gnf is not part of the written up model. LAMBDA is. 
  # In current write-up. Each collection has the SAME underlying genotype frequencies. There is no distinction between 'sample' and 'population' genotype frequencies.
  # gnf_plambda = log(ddirichlet(npars$gnfp$gnf_lambda, (npars$gnfp$gnf_ws      *npars$gnfp$gnf_apar)    )) #prior prob of 'lambda'
  #gnf             = npars$gnfp$gnf
  #gnf             = myapply(X=gnf, MARGIN=2, FUN=normalise)
  #pop_gn_freqs    = apply(gnf, 1, mean)

  if(nocl>1) {
    gnf_lambda    = npars$gnfp$gnf_lambda
    if(flags$use_null_class) gnf_sum = (1-npars$null_prop) else gnf_sum=1
    if(check_dat(data=gnf_lambda, precision=1e-8, sum=gnf_sum)) {cat("In calc_newprior. Problem with genotype frequencies.\n"); browser();}
    pr_gnf_lambda = ld_dirichlet(data=normalise(gnf_lambda), alpha=(npars$gnfp$gnf_ws * npars$gnfp$gnf_theta)) #prior prob of 'lambda'
    #npars$gnfp$gnf_apar cannot be 1 due to the definition of the beta distribution! What to do about this?
    gnf_pralpha   = dbeta(npars$gnfp$gnf_apar  ,  npars$gnfp$gnf_hp_shape1, npars$gnfp$gnf_hp_shape2, log=T)  #prior prob of 'alpha'
    #gnf_prval    = gnf_call_prior + gnf_plambda + gnf_pralpha;
   #nullp_mv      = mean_var_to_standard_pars(distribution="gamma", mean=1/1000, var=0.002^2, verbose=F)
   #nullf_prval   = sum(dgamma(npars$gnfp$nullf, scale=nullp_mv["scale"], shape=nullp_mv["shape"], log=T))
   #nullp_mv      = mean_var_to_standard_pars(distribution="beta", mean=1/1000, var=0.001^2, verbose=F)
   #nullf_prval   = sum(dbeta(npars$gnfp$nullf, shape1=nullp_mv["shape1"], shape2=nullp_mv["shape2"], log=T))
    if( flags$null_prior) nullf_prval = sum(dnorm(npars$gnfp$nullf, mean=1/1000, sd=0.002, log=T)+log(1.9)) #Currently +log(1.9) is a hack to mimic truncating the nonmal
    if(!flags$null_prior) gnf_prval   = pr_gnf_lambda + gnf_pralpha;
    if( flags$null_prior) gnf_prval   = pr_gnf_lambda + gnf_pralpha + nullf_prval;
  } else
  {
    gnf_prval     = 0 #This is a logged probability, need to do this as 'alpha' is meaningless for nocl==1
    gnf_pralpha   = 0
    pr_gnf_lambda = 0
  }
  if(flags$DEBUG>2.5) {
    cat("\tlprgnf make-up is: gnf_plambda ", pr_gnf_lambda, ",\n", "\tgnf_pralpha ", gnf_pralpha, "\n", sep="");
  }
  if(flags$DEBUG>2.5) {
    cat("\tlprior make-up is: smc_prval: ", (lsmc_prval), ", cld_prval ", (lcld_prval), ", clv_prval ", clv_prval, ",\n",
        "\tassoc_prval ", (lassoc_prval), ", gnf_prval ", gnf_prval, "\n", sep="");
  }
  if(lsmc_prval< -1E250 | lcld_prval< -1E250 | lassoc_prval< -1E250 | clv_prval < -1E200 | gnf_prval < -1E200) {
    if(flags$DEBUG>0.9) cat("Warning! Very unlikely prior value of suggested move. on iteration", flags$iter, "\n");
    if(flags$DEBUG>2.5) browser();
    prval = -1E300
  } else {
    ## THIS is the key line which sets the prior value. This is then passed to: objects$likelihood$n_prior_brkdwn$ltotal_prior
    ## The value: objects$likelihood$n_prior_brkdwn$ltotal_prior is then passed to: objects$likelihood$n_prior
    ## This is then used to evaluate the move with: accept_or_reject(... old_pr = objects$likelihood$o_prior, new_pr = objects$likelihood$n_prior...)
    prval = lassoc_prval + lsmc_prval + lcld_prval + gnf_prval + clv_prval;
  }
  if(!log) cat("Program Warning - prior probabilities can only be given logged.\n");
  if(is.na(prval) | abs(prval)==Inf) {
    cat("Error in calc_newprior - Invalid prior of parameters:", prval, "\n"); 
    cat("\tsmc=", exp(lsmc_prval), "cld=",exp(lcld_prval), "clv=",clv_prval, "asoc=", exp(assoc_prval), "l(gnf)=",gnf_prval, "\n"); 
    if(flags$DEBUG>1.7) browser();
    return(-1E300)
    browser();
  }
  if(length(prval)!=1) {cat("In calc_newprior. Prior is not a _single_ number!\n"); browser();}
 #rv = list("lsmc"=(lsmc_prval), "lcld"=(lcld_prval), "lassoc"=(lassoc_prval), "llambda_g_ws" = pr_gnf_lambda, "lgnf"=gnf_prval, "lclv"=clv_prval, "ltotal_prior"=prval)
  rv = prior_ret_val(real_priors=T, lsmc=(lsmc_prval), lcld=(lcld_prval), lassoc=(lassoc_prval), llambda_g_ws = pr_gnf_lambda, lgnf=gnf_prval, lclv=clv_prval, ltotal_prior=prval)
  return(rv);
}

prior_ret_val=function(real_priors=T, lsmc, lcld, lassoc, llambda_g_ws, lgnf, lclv, ltotal_prior)
{
  if( real_priors) rv = list("lsmc"=lsmc, "lcld"=lcld, "lassoc"=lassoc, "llambda_g_ws" = llambda_g_ws, "lgnf"=lgnf, "lclv"=lclv, "ltotal_prior"=ltotal_prior)
  if(!real_priors) rv = list("lsmc"=0, "lcld"=0, "lassoc"=0, "llambda_g_ws" = 0, "lgnf"=0, "lclv"=0, "ltotal_prior"=0)
  return(rv)
}

##This function uses bic to test for association based on a 1 parameter difference between the null and additive assocation model
test_for_association=function(additive_assoc_lklihd, no_assoc_lklihd, no_data_points, verbose=F)
{
  log_bf = additive_assoc_lklihd-no_assoc_lklihd - 1*log(no_data_points)
  if(verbose) cat("Testing for association.\n");
  if(length(log_bf)>1) browser()
  return(log_bf)
}

#This only calculates the model evidence of the INTENSITY data using the simple non-hierarchical model. No association etc. taken into account here.
calc_evidence=function(fit, separate_collections=F, set_point = NULL, type="bic", new=F)
{
  if(length(fit)==1){ if(sum(names(fit[[1]])[1:5]==c("dat","opars","npars","ninferred","oinferred"))==5) fit = fit[[1]]}
  if(is.null(fit[[1]])) return(NA)
  nocl           = fit$opars$nocl
  ind_p_coll     = colSums(fit$dat)
  if( separate_collections) {
    weights      = normalise(ind_p_coll, sum=1)
    no_pars_vec  = calc_pars_per_coll(nocl=nocl, weights=weights)
    if(!is.null(set_point)) {
      check = (set_point == "hier_mchoice"); if(fit$flags$DEBUG>0.1 & !check) {cat("Error in calc_evidence.\n"); browser();}
      l_lkhds      = fit$lstore$hier_mchoice$coll_spec_liklihd    
    } else                  {
      l_lkhds = calc_intens_lkld(dat=fit$dat, best_call=!fit$flags$sum_calls, test=F, objects=fit, new=new, colls_separate=T)
      # fit = best_fit; l1 = calc_intens_lkld(dat=fit$dat, best_call=!fit$flags$sum_calls, test=F, objects=fit, new=F, colls_separate=T)
      #                 l2 = fit$lstore$hier_mchoice$coll_spec_liklihd    
    }
    if(type=="bic") {
      evs      = l_lkhds - no_pars_vec*log(ind_p_coll)
      evidence = sum(evs)
    }
    if(type=="aic") {
      evs      = l_lkhds - no_pars_vec*log(2)
      evidence = sum(evs)
    }
  }
  if(!separate_collections) {
    no_pars        = calc_simple_no_pars(nocl)
    if(       is.null(set_point)   ) { l_lkhd = fit$likelihood$o_liklihd
    } else { if(set_point == "simple") l_lkhd = fit$lstore$simple_mchoice$o_liklihd else { 
         cat("Don't know what likelihood to use to calculate the evidence!.\n"); browser();
       }
    } 
    no_data_points = sum(ind_p_coll)
    if(type=="bic")
      evidence   = get_bic(likelihood=l_lkhd, no_pars=no_pars, no_data_points=no_data_points)
    if(type=="aic")
      evidence   = get_aic(likelihood=l_lkhd, deg_free=no_pars, k=2)
  }
  if(length(evidence)>0) { if(!(!is.na(evidence) & abs(evidence)<Inf)) {cat("Problem in calc_evidence.\n"); browser();} } else {cat("No evidence has been calculated.\n"); browser();}
  return(evidence)
}

#My current understanding is that k=2 (where with bic k=log(n)) and deg_free is the number of parameters.
get_aic=function(likelihood, deg_free, k = 2)
{
   aic = 2*likelihood - (k * deg_free)
   return(aic)
}
get_bic=function(likelihood, no_pars, no_data_points)
{
   bic = likelihood - no_pars*log(no_data_points)
   return(bic)
}

calc_evidence_old=function(nocl, likelihood, no_data_points)
{
  no_pars    = calc_simple_no_pars(nocl)
  bic_ev     = likelihood - (no_pars*log(no_data_points))
  return(bic_ev)
}

calc_pars_per_coll=function(nocl, weights, sum_calls=T, no_inds=NULL)
{
  if( sum_calls) pars        = c("means"=nocl, "mean_hpars"=0, "vars" = nocl, "nar_hpars" = 0, "gnf" = nocl-1) 
  if(!sum_calls) pars        = c("means"=nocl, "mean_hpars"=0, "vars" = nocl, "nar_hpars" = 0, "gnf" = nocl-1, "calls" = no_inds) 
  pars_mat    = matrix(pars, ncol=length(weights), nrow = length(pars), dimnames=list(names(pars),names(weights)));
  pars_mat["gnf",] = pars_mat["gnf",] * weights
  no_pars_vec = colSums(pars_mat);
  #no_gnf_pars = no of parameters = (no classes -1) * weight of this collection 
  #Where the weight of each collection is proportional to the number of individuals in that collection and all the weights add up to 1. This intuitively tries to ensure that _total_ bic penalty is correct. 
  return(no_pars_vec)
}

calc_simple_no_pars=function(nocl)
{
  pars        = c("means"=nocl, "mean_hpars"=0, "vars" = nocl, "nar_hpars" = 0, "gnf" = nocl-1) 
  no_pars     = sum(pars);
  return(no_pars)
}


