#diagnostics
#sim10 = read.table("sim_study_stseed=10_run=1 (copy).txt", as.is=T, header=T)
# h_calls = check_calls(ob = stpt_states[[3]], dat=dat, cnv_types=cnv_types, data_array=data_array)
check_calls=function(ob, dat=NULL, cnv_types=NULL, data_array=NULL, ung_dat=NULL)
{
  if(is.null(dat))            dat = ob$dat
  if(is.null(dim(ob$dat))) ob$dat = dat
  if(is.null(dim(ob$dat)) & is.null(dat)) cat("Warning, no matrix of gridded values.\n")
  
  nocl = ob$opars$nocl
  noco = ob$opars$noco
  if(!is.null(data_array)) if(noco != length(data_array)) {cat("Warning! Unclear on number of collections.\n"); browser();}
  # This merely makes calls for GRID VALUES. 
  grid_hcalls = make_grid_calls(objects=ob, use_null_freq=F)

  # WARNING - this version is uses probe_means, a PCA probe-summary is not yet implemented
  # Simple robust Version for now;
  h_calls = as.list(rep("", noco))
  for(coll in 1:noco)
  {
    if(is.null(ung_dat)) {
      cat("Warning - 1-d data summary not passed, so simply using probe means.\n");
      if(is.null(data_array)) {cat("Cannot proceed. No data on each individual to map grid_point calls to people.\n"); browser();}
      one_dsmry       = rowMeans(data_array[1,coll][[1]])
      rmeans          = range(one_dsmry)
      one_dsmry       = one_dsmry - min(one_dsmry)  
      one_dsmry       = one_dsmry / (rmeans[2]-rmeans[1]) * 0.1 ##0.1 is the data_range that the priors are optimised for  
      one_dsmry       = one_dsmry - 0.05
    } else               {
      one_dsmry     = ung_dat[[coll]]
    }
    grid_breaks     = ob$opars$grid_breaks
    ind_grid_vals   = apply(X=matrix(one_dsmry, nrow=length(one_dsmry), ncol=1), MARGIN=1, FUN=data_boxer, grid_breaks=grid_breaks)
    h_calls[[coll]] = grid_hcalls[ind_grid_vals]
    #display_correlations(h_calls=h_calls, cnv_types=cnv_types, nocl=nocl)
  }  
  if(ob$flags$DEBUG>1.5) if(sum(is.na(unlist(h_calls)))==length(unlist(h_calls))) { cat("Warning - no hard calls were makeable.\n"); browser();}
  return(h_calls)
}


display_correlations=function(h_calls, cnv_types, nocl, tr_nocl, write=F, filetag="temp", index=NULL)
{
  fn = paste(filetag, ".txt", sep="");
  preamble = paste("bic_chosen nocl:", nocl, ", true_nocl=", tr_nocl, ". Correlation with true calls: ", sep="")
  if(write) write.table(preamble, file=fn, append = file.exists(fn), quote=F, row.names=F, col.names=F, eol="  ")
  for(coll in 1:length(data_array))
  {
    this_cor = calls_cor(h_calls=h_calls, cnv_types=cnv_types, nocl=nocl)
    cat("Correlation^2 of calls with simulated cnv_types in collection", coll, ":", this_cor, "\n");
    if(write) write.table(this_cor, file=fn, append = file.exists(fn), quote=F, row.names=F, col.names=F, eol="  ")
  }
  if(write) write.table("\n", file=fn, append = file.exists(fn), quote=F, row.names=F, col.names=F, eol="  ")
}


detailed_sim_output=function(objects, sim_pars, simdat, sim_index, filetag="tmp_details_", filename=NULL, tr_nocl, h_calls, ung_dat)
{
  # Write detailed match file. For each CNV use 2 lines? a long vector of numbers (with headers)
  # Truth on the first row, inferred on the second row. (Blank lines between?)
  # Give: true_nocl , bic_nocl
  #       true_betas, bic_best_betas
  #       concordance with bic_best
  #       true_gnf  , bic_best gnf #These vectors will often have different lengths/dimensions!
  #       true_clm  , bic_best clm #These vectors will often have different lengths/dimensions!
  #          - Probably start with 'empirical' means basesd on true cnv types
  #       true_clv  , bic_best clv #These vectors will often have different lengths/dimensions!
  #          - Probably start with 'empirical' variances basesd on true cnv types

 #cnv_types        = simdat$cnv_types; 
 #noco             = length(cnv_types)
  rv        = mergeNocos(infList=h_calls, cnv_types=simdat$cnv_types)
  noco      = rv$noco
  cnv_types = rv$cnv_types
  bic_nocl         = objects$opars$nocl
  sim_priors       = simdat$priors_used; 
  concordance      = calc_sim_concordance(h_calls=h_calls, cnv_types=cnv_types)
  conc_names       = paste(rep("conc", noco), 1:noco, sep="")
  r_squared        = calls_cor(h_calls=h_calls, cnv_types=cnv_types, nocl=bic_nocl)
  r_sq_names       = paste(rep("cors", noco), 1:noco, sep="")
  tr_betas         = c(sim_priors$beta0, sim_priors$beta1)
  names(tr_betas)  = c("trbeta0", "trbeta1")
  if(DEBUG>3) { #This discrepency can be caused by the simulated data being simulated with 'asociation=FALSE'
    tr_betas2        = c(sim_pars$betas["beta0"], sim_pars$betas["beta1"])
    check_trbetas    = sum(abs(tr_betas-tr_betas2)<1e-7)==length(tr_betas)
    if(!check_trbetas) {cat("Warning - disagreement in what betas used in the sim study!\n"); if(objects$flags$DEBUG>1.5) browser()}
  }
  inf_betas        = objects$opars$betas
  names(inf_betas) = c("infbeta0", "infbeta1")

 #tr_emp_gnf       = make_emp_gnf(cnv_types=cnv_types, noco=noco, tr_nocl=tr_nocl, use_h_calls=T)
 #inf_gnf          = objects$opars$gnfp$gnf
 #max_nocls_gnf    = max(nrow(inf_gnf),nrow(tr_emp_gnf))

  inf_clms         = make_clm(smcs=objects$opars$smc , clds=objects$opars$cld, noco=noco, nocl=bic_nocl)
  tr_clms          = make_emp_moments(cnv_types=cnv_types, grid_dat = objects$dat, sim_dat=ung_dat, tr_nocl=tr_nocl, moment="mean")
  max_nocls_m      = max(ncol(inf_clms),ncol(tr_clms))

  inf_clvs         = objects$opars$clv
  tr_clvs          = make_emp_moments(cnv_types=cnv_types, grid_dat = objects$dat, sim_dat=ung_dat, tr_nocl=tr_nocl, moment="var")
  max_nocls_v      = max(ncol(inf_clvs),ncol(tr_clvs))
  check            = (max_nocls_v==max_nocls_m) & (max_nocls_m  ==(max(bic_nocl,tr_nocl)) )
  if(!check) {cat("Error - In detailed sim output. Unclear how many classes to report for.\n"); browser();}
  if(tr_nocl> bic_nocl)
  {
    inf_clms = cbind(inf_clms, matrix(NA, nrow=noco, ncol=(tr_nocl-bic_nocl)))
    inf_clvs = cbind(inf_clvs, matrix(NA, nrow=noco, ncol=(tr_nocl-bic_nocl)))
   #inf_gnf  = rbind(inf_gnf , matrix(NA, ncol=noco, nrow=(tr_nocl-bic_nocl)))
  }
  if(tr_nocl< bic_nocl)
  {
    tr_clms   = cbind(tr_clms   , matrix(NA, nrow=noco, ncol=(bic_nocl-tr_nocl)))
    tr_clvs   = cbind(tr_clvs   , matrix(NA, nrow=noco, ncol=(bic_nocl-tr_nocl)))
   #tr_emp_gnf= rbind(tr_emp_gnf, matrix(NA, ncol=noco, nrow=(bic_nocl-tr_nocl)))
  }
  if(tr_nocl==bic_nocl)
  {  }
  inf_clm_vec = as.vector(inf_clms)
  inf_clv_vec = as.vector(inf_clvs)
 #inf_gnf_vec = as.vector(inf_gnf )
   tr_clm_vec = as.vector( tr_clms) #Note that these currently come in a DIFFERENT ORDER
   tr_clv_vec = as.vector( tr_clvs)
  #tr_gnf_vec = as.vector(tr_emp_gnf) #to clm and clvs, just because that's how it is in the program

  moms_length = max_nocls_m * noco
  names(tr_clm_vec)   = paste(rep("clm", moms_length), 1:moms_length, sep="")
  names(tr_clv_vec)   = paste(rep("clv", moms_length), 1:moms_length, sep="")
 #names(tr_gnf_vec)   = paste(rep("gnf", moms_length), 1:moms_length, sep="")

  write_obj_tr        = matrix(c(tr_nocl , tr_betas , tr_clm_vec , tr_clv_vec  ), nrow=1)
  write_obj_inf       = matrix(c(bic_nocl, inf_betas, inf_clm_vec, inf_clv_vec ), nrow=1)
  write_obj           = rbind(write_obj_tr, write_obj_inf)
  write_obj           = round(write_obj, digits=6)
  colnames_write_obj  = c("nocl", "beta0", "beta1", names(tr_clm_vec), names(tr_clv_vec))
  colnames(write_obj) = colnames_write_obj

  if(sim_index==1) {
    if(is.null(filename)) fn = pick_filename(filetag) else fn = filename
    write.table(format(write_obj), file=fn, append = F, quote=F, row.names=F, col.names=T, eol="\n")
  } else
  {
    fn = filename
    write.table(format(write_obj), file=fn, append = T, quote=F, row.names=F, col.names=F, eol="\n")
  }
  return(fn)
}

# 'make empirical genotype frequencies': This calculates the true allele frequencies based on the (true) cnv_types.
make_emp_gnf=function(cnv_types, noco, tr_nocl, use_h_calls=T)
{
  if(use_h_calls)
  {
    gnf = matrix(NA, nrow=tr_nocl, ncol=noco)
    for(cl in 1:tr_nocl)
    {
      for(coll in 1:noco)
      {
        this_cl_inds = which(cnv_types[[coll]]==cl)
       #this_cl_dat  = sim_dat[[coll]][this_cl_inds]
        gnf[cl,coll] = length(this_cl_inds)/length(cnv_types[[coll]])
      }
    }
  }
  return(gnf)
}

#This function calculates the (empirical) mean or variance of the PCA (or otherwise transformed 1-d) data within each cluster.
#This is (by default anyway) collection specific
make_emp_moments=function(cnv_types, sim_dat, tr_nocl, moment, h_calls=T, grid_dat)
{
  #rv        = mergeNocos(infList=h_calls, cnv_types=cnv_types)
  #noco      = rv$noco
  #cnv_types = rv$cnv_types

  noco = length(cnv_types)
  if(h_calls)
  {
    cl_moments = matrix(NA, nrow=noco, ncol=tr_nocl)
    for(cl in 1:tr_nocl)
    {
      for(coll in 1:noco)
      {
        this_cl_inds = which(cnv_types[[coll]]==cl)
        this_cl_dat  = sim_dat[[coll]][this_cl_inds]
        if(length(this_cl_inds)>1) {
          if(moment=="mean") this_cl_moment  = mean(this_cl_dat)
          if(moment=="var" ) this_cl_moment  =  var(this_cl_dat)
        } else
        {
          if(moment=="var" )                           this_cl_moment  =  NA
          if(moment=="mean" & length(this_cl_inds)==0) this_cl_moment  =  NA
          if(moment=="mean" & length(this_cl_inds)==1) this_cl_moment  =  mean(this_cl_dat)
        }
        cl_moments[coll, cl] = this_cl_moment
      }
    }
  } else { #Using soft_calls (ie. expectations)
    cat("In make_emp_moments. Soft_calls version not implemented yet.\n");
    browser();
  }
  return(cl_moments)
}

mergeNocos=function(infList, cnv_types)
{
  trNoco =length(cnv_types)
  infNoco=length(infList)
  if(trNoco!=infNoco) {
    if(infNoco==1) {
      ulCnvTypes = unlist(cnv_types)
      cnv_types  = as.list(NA)
      cnv_types[[1]] = ulCnvTypes
    } else         {
      cat("  Warning! sim and inference number of collections disagrees, why is this?\n")
      if(DEBUG>1.1) browser()
    }
  }
  noco   = infNoco
  rv = list("noco"=noco, "cnv_types"=cnv_types)
}

calc_sim_concordance=function(h_calls, cnv_types, DEBUG=1.1)
{
  rv        = mergeNocos(infList=h_calls, cnv_types=cnv_types)
  noco      = rv$noco
  cnv_types = rv$cnv_types
  conc   = rep(-1, noco)
  agrees = rep(-1, noco)
  totals = rep(-1, noco)
  for(coll in 1:noco) {
    agrees[coll] = sum(h_calls[[coll]]==cnv_types[[coll]])
    totals[coll] = length(cnv_types[[coll]])
    conc[coll]   = agrees[coll]/totals[coll]
  }
  names(conc) = paste(rep("conc", ), 1:noco)
  return(conc)
}

# debug(check_calls)
# debug(data_boxer)
data_boxer=function(data_pt, grid_breaks)
{
 gp = which(grid_breaks>data_pt)[1]-1
 return(gp)
}

check_cols_same=function(objects, initialise=F, mv=NA, precision=1e-10)
{
  if(is.null(rownames(objects$opars$clv))) {cat("clv has lost its rownames.\n"); browser();}

  nclv          = objects$npars$clv               ; ncld          = objects$npars$cld               ;
  nclvs_same    = sum(abs(apply(nclv, 2, diff))<precision)    ; nclds_same    = sum(abs(apply(ncld, 2, diff))<precision)    ;
  no_nclv_diffs = (nrow(nclv)-1)*ncol(nclv)       ; no_ncld_diffs = (nrow(ncld)-1)*ncol(ncld)       ;
  colls_diff = (nclvs_same==no_nclv_diffs) & (nclds_same==no_ncld_diffs)
  if(objects$flags$DEBUG>0.1 & (objects$flags$model=="simple" |!objects$flags$use_hierarchy) & !colls_diff) {
    cat("Error. All collections should be the same with the non-hierarchical model.\n");
    if(objects$flags$iter<=1 | initialise) cat("Problem is probably in initialisation.\n")
    if(objects$flags$iter >1) cat("Current move type is:", mv, ".\n")
    browser()
  }
}


#smc_min_val has been set to 0.055 because when the nocl is completely wrong (particular when tr_nocl==1) 
#sometimes the RIGHT model fit involves smc>0
check_clm=function(objects, smcs=NULL, dsts=NULL, in_clm=NULL, move="not_specified_function", cur=F, average=F, smc_min_val=0.0505, extra_added=F, stop=F, precision=1e-8)
{
   #extra_added is a flag to say whether or not the fixed/minimum clds have already been included in the clm/clds passed to this function
   if(!objects$flags$use_mpar_additions) extra_added=T; #This is a simple way of ensuring that they are not added on multiple times.
   if(is.null(in_clm) & is.null(smcs) & is.null(dsts)) {if(cur) in_clm = objects$opars$hkclm else in_clm = objects$npars$hkclm; extra_added=T}
   if(!is.null(in_clm)) if(is.na(range(in_clm)[1])) {      cat("In check clm. NAs in these class means\n"       ); browser();}
   if( is.null(in_clm)) if(is.na(range(c(smcs,dsts))[1])) {cat("In check clm. NAs in these class descriptions\n"); browser();}
   dat  = objects$dat
   nocl = objects$opars$nocl
   noco = objects$opars$noco
   if( is.null(in_clm)) {if(sum(         dsts         <0)>0) {cat("In check_clm. Can't have negative dsts1.\n"); browser();}}
   if(!average & objects$flags$DEBUG>1.5) {
     if( is.null(in_clm)) {if(nrow(dsts  )!=noco | ncol(dsts  )!=(nocl-1)) {cat("In check_clm. dimensions of cld seem to be wrong.\n"); browser();}}
     if(!is.null(in_clm)) {if(nrow(in_clm)!=noco | ncol(in_clm)!=(nocl  )) {cat("In check_clm. dimensions of clm seem to be wrong.\n"); browser();}}
     if(!is.null(in_clm)) {if(sum(apply(in_clm, 1, diff)<0)>0            ) {cat("In check_clm. Can't have negative dsts2.\n"); browser();}}
     if(objects$flags$DEBUG>2.5) {
       if( is.null(in_clm)) {if(is.null(rownames(dsts  ))) {cat("In check_clm. cld    has no rownames.\n"); browser();}}
       if(!is.null(in_clm)) {if(is.null(rownames(in_clm))) {cat("In check_clm. in_clm has no rownames.\n"); browser();}}
     }
   }
   if(average & !is.null(in_clm)) {if( sum(diff(in_clm)<0)   >0) {cat("In check_clm. Can't have negative dsts3.\n"); browser();}}
   q1          = cumsum(dat) #These are used to find certain quantiles of the data 
   no_inds     = sum(dat)
   grid_vals   = as.numeric(rownames(dat))
   extreme_gvs = calc_high_low_grid_vals(objects=objects)


     if(is.null(in_clm)) {
       if(!average) {
         clm = make_clm(smcs=smcs, clds=dsts, noco=noco, nocl=nocl)
       } else       {
         clm = cumsum(c(smcs,dsts))
         clm=matrix(rep(clm, noco), ncol=nocl, byrow=T)
       }
     } else           {
       if(!average) {
         #clm = apply(as.matrix(in_clm), 2, mean)
         clm=in_clm
         if(noco==1) clm=in_clm
       } else       {
         clm=matrix(rep(in_clm, noco), ncol=nocl, byrow=T)
       }
     }
     if(!is.null(in_clm)) {if(length(clm)<length(in_clm)) {cat("In check_clm. Error, have processed in_clm wrong\n"); browser();}}
     if(noco==1 & length(clm)           != objects$opars$nocl) {cat("Error! Have miscalculated class means.\n"); browser();}
     if(noco >1){ if(nocl>1) { if(ncol(clm) != objects$opars$nocl) {cat("Error! Have miscalculated class means.\n"); browser();}}}
     if(noco >1){ if(nocl==1 & length(clm)  != noco              ) {cat("Error! Have miscalculated class means.\n"); browser();}}
     if(!extra_added) {
       new_moms = apply_edge_effects_hack(mean=clm, var = objects$npars$clv, cl="all", vals=grid_vals, flags = objects$flags)
       clm      = new_moms$new_mean
       #browser();
     }

     #Here is where the checks for implausible values (rather than misconstructed objects) occur:
     if(( (min(clm ) >= grid_vals[extreme_gvs["low_gv"] ]-precision )
         &(max(clm ) <= grid_vals[extreme_gvs["high_gv"]]+precision )
         &(max(clm[1]) <= smc_min_val+precision | nocl==1)
        ) | objects$flags$DEBUG<1.8
     ) 
     {
       reject=F
     } else
     {
       reject=T
       if( cur)               cat("Current state of chain is silly. (In ", move, ")\n", sep="");
       if(!cur & nocl>1 & (objects$flags$DEBUG>5 | stop))  cat("New state of chain is silly. (In ", move, ")\n", sep="");
       if(objects$flags$DEBUG>1.5 & (cur | stop)) browser();
     }
  return(reject)
}


check_moves_made = function(move_records)
{
  mv_names = move_records$mv_names
  accept   = move_records$accept
  mv_types = unique(as.vector(mv_names))
  var_inds    = which(mv_names=="vars", arr.ind=T)
  if(length(var_inds)>0) {
    colnames(var_inds)    = c("nocl", "iteration")
    #var_inds[,1] = nocls[var_inds[,1]]
    #var_inds = var_inds[,2:1]
    no_new_vars    = sum(accept[var_inds   ])
    accepted_var_moves    = which((accept[var_inds   ])==1, arr.ind=T)
    rejected_var_moves    = which((accept[var_inds   ])==0, arr.ind=T)
       var_inds[accepted_var_moves   ,]
    cat("The number of times alg. tried to change variances is:", nrow(var_inds)   , "\n")
    cat("The number of times variances were actually changed was:", no_new_vars   , "\n")
    if(length(accepted_var_moves)>0) vaiter = var_inds[accepted_var_moves[1],"iteration"]
    if(length(rejected_var_moves)>0) vriter = var_inds[rejected_var_moves[1],"iteration"]
    if(no_new_vars>0) cat("First var acceptance at: ", accepted_var_moves[1], "th attempt (iter ", vaiter, ")\n", sep="")
    if(no_new_vars>0) cat("First var rejection  at: ", rejected_var_moves[1], "th attempt (iter ", vriter, ")\n", sep="")
  } else {
    cat("The number of times alg. tried to change variances is:", 0, "\n")
    cat("Total number of iters = ", no_iters, "\n");
  }
  cat("\n");

  maxgnf_inds = which(mv_names=="comb_max_gnf", arr.ind=T)
  if(length(maxgnf_inds)>0) {
    colnames(maxgnf_inds) = c("nocl", "iteration")
    no_new_maxgnfs = sum(accept[maxgnf_inds])
    accepted_maxgnf_moves = which((accept[maxgnf_inds])==1, arr.ind=T)
    rejected_maxgnf_moves = which((accept[maxgnf_inds])==0, arr.ind=T)
    maxgnf_inds[accepted_maxgnf_moves,]
  
    cat("The number of times alg. tried to change gnf is:", nrow(maxgnf_inds), "\n")
    cat("The number of times gnf was actually changed was:", no_new_maxgnfs, "\n")
    if(length(accepted_maxgnf_moves)>0) maiter = maxgnf_inds[accepted_maxgnf_moves[1],"iteration"]
    if(length(rejected_maxgnf_moves)>0) mriter = maxgnf_inds[rejected_maxgnf_moves[1],"iteration"]
    if(length(accepted_maxgnf_moves)>0) cat("First maxgnf acceptance at: ", accepted_maxgnf_moves[1], "th attempt (iter ", maiter, ")\n", sep="")
    if(length(rejected_maxgnf_moves)>0) cat("First maxgnf rejection  at: ", rejected_maxgnf_moves[1], "th attempt (iter ", mriter, ")\n", sep="")
  } else {
    cat("The number of times alg. tried to change gnf is:", 0, "\n")
    cat("Total number of iters = ", no_iters, "\n");
  }
}

make_colnames_for_details=function(fn)
{
  nrows   = length(readLines(fn))
  no_cols = 0
  con     = file(fn)
  open(con, open="r")
  for(cnv in 1:(nrows/2))
  {
    this_cnv_det = read.table(file=con, as.is=T, header=F, fill=F, skip=0, nrow=2)
    no_cols      = max(no_cols, ncol(this_cnv_det))
  }
  fake_colnames = paste(rep("col", no_cols), 1:no_cols, sep="")
  close(con)
  return(fake_colnames)
}

# Make moments (ie. means and variances of clusters) from details object
make_moms_from_det=function(cnv_det, nocl, noco, type)
{
   tr_moms = matrix(NA, nrow=noco, ncol=nocl)
  inf_moms = matrix(NA, nrow=noco, ncol=nocl)
  mom_indices       = which(substr(colnames(cnv_det), start=0, stop=3)==type)
  for(coll in 1:noco)
    for(cl in 1:nocl)
    {
      if(type!="gnf") mom_index         = (cl  -1)*noco + coll
      if(type=="gnf") mom_index         = (coll-1)*nocl + cl
       tr_moms[coll,cl] = cnv_det[1,mom_indices[mom_index]]
      inf_moms[coll,cl] = cnv_det[2,mom_indices[mom_index]]
    }  
  rownames( tr_moms) = paste(rep("coll", noco), 1:noco, sep="")
  colnames( tr_moms) = paste(rep(type  , nocl), 1:nocl, sep="")
  rownames(inf_moms) = rownames( tr_moms)
  colnames(inf_moms) = colnames( tr_moms)
  moms        = list( tr_moms ,  inf_moms )
  names(moms) =    c("tr_moms", "inf_moms")
  return(moms)
}

make_det_colnames=function(cnv_det, noco, details_hdr)
{
    #Want to create: colnames(cnv_det)
    upto     = 0
    colnames(cnv_det)[(upto+1):(upto+3)] = details_hdr[1:3]
    upto     = upto + 3
    no_cls   = max(cnv_det[,"nocl"])
    no_pars  = no_cls*noco
    colnames(cnv_det)[(upto+1):(upto+no_pars)] = paste(rep("clms", no_pars), 1:no_pars, sep="")
    upto     = upto + no_pars
    colnames(cnv_det)[(upto+1):(upto+no_pars)] = paste(rep("clvs", no_pars), 1:no_pars, sep="")
    upto     = upto + no_pars
    colnames(cnv_det)[(upto+1):(upto+no_pars)] = paste(rep("gnf" , no_pars), 1:no_pars, sep="")
    upto     = upto + no_pars
  return(cnv_det)
}

visualise_data=function(data_array, ung_dat, g_dat, noco=1, cnv_name=NA, cnv_index=NA)
{
    cat("Program run in data visualisation mode.\n")
    par(mfcol=c(2,2))
    for(coll in 1:length(ung_dat))
       hist(ung_dat[[coll]], n=100, col="lightgrey")
    hist(unlist(ung_dat),  n=100, col="lightgrey")
    browser()
    look_at_data_array(data_array=data_array, noco=1, cnv_name=cnv_name, cnv_index=cnv_index)
    da2 = remove_extremes(data_array=data_array, no_probes_to_remove=20, noco=1, remove_two=F, rinds=T, rprobes=F)
    cat("Have removed most extreme probes.\n")  
    look_at_data_array(data_array=da2, noco=1, cnv_name=cnv_name, cnv_index=cnv_index)
}

#This function tries to locate the files containing INFERRED MODEL FITS
get_stpt_states=function(seed=1, cnv_index=NULL, cnv_name=NULL, no_runs=NA)
{
  # if(!real_data)stpts_dir = paste(stpts_dir, "sim_study_runs/", sep="")
  # stpts_fn = paste(stpts_dir, "seed=", seed, "_cnv=", cnv_index, "_no_runs=", no_runs, ".RData", sep="")
  if(is.null(cnv_name)) cnv_name = cnv_index
  stpts_fn = get_coutput_fns(cnv_name=cnv_name)$stpts_fn
  if(file.exists(stpts_fn)) {
    x  = load(stpts_fn) 
    rv = model_fits$stpt_states
  } else {
    rv = "stpt_states_not_written"
  }
  return(rv)
}

summarise_these_model_fits=function(cnv_index=3, seed=1, no_runs=500, write_ls=F, tr_nocl, inf_nocl, png=F, write_eps, 
            image_ftag=paste("sim_calls/", "Stpt_Likelihoods_", sep=""), stpts_dir, real_data)
{
  stpt_states = get_stpt_states(stpts_dir = stpts_dir, seed=seed, cnv_index=cnv_index, no_runs=no_runs)
  nocls       = nrow(stpt_states)
  stpts       = ncol(stpt_states)
  mod_lhds    = matrix(NA, nocls, stpts)
  log_bic_evs = matrix(NA, nocls, stpts)
  no_inds     = sum(stpt_states[1,1][[1]]$dat)
  for(nocl in 1:nocls)
  {
    for(stpt_ind in 1:stpts)
    {
     #tmp = stpt_states[nocl,stpt_ind][[1]]$likelihood$o_intsty
      tmp = stpt_states[nocl,stpt_ind][[1]]$likelihood$simple_liklihd
      if(!is.null(tmp)) {
           mod_lhds[nocl,stpt_ind] = tmp
        log_bic_evs[nocl,stpt_ind] = calc_evidence_old(nocl=nocl, likelihood=mod_lhds[nocl,stpt_ind], no_data_points=no_inds)
      }
    }
  }
  
  png_fn = paste(image_ftag, "seed=", seed, "_cnv_index=", cnv_index, ".png", sep="")
  eps_fn = paste(image_ftag, "seed=", seed, "_cnv_index=", cnv_index, ".eps", sep="")
  if(eps&png){cat("Can't yet output both pngs and eps files.\n"); browser();}
  if(png) png(file=png_fn, width=1200, height=800)
  par(mfcol=c(1,2))
  main1 = paste("CNV index: ", cnv_index, sep="")
  main2 = paste("True no. Classes = ", tr_nocl, ", inferred = ", inf_nocl, sep="")
  ylab1 = paste("Log Probability of Data under Simple Model", sep="")
  ylab2 = paste("BIC Model Evidence", sep="")
  #image(x=1:stpts, y=1:nocls, z=t(mod_lhds), main=main1)  
  plot_l_curves(lkhd=mod_lhds   , main=main1, ylab=ylab1, tr_nocl=tr_nocl, stpts=stpts)
  plot_l_curves(lkhd=log_bic_evs, main=main2, ylab=ylab2, tr_nocl=tr_nocl, stpts=stpts)
  if(eps) dev.copy2eps(file=eps_fn)
  if(png) dev.off()
  return(invisible(mod_lhds))
}

# plot_l_curves(lkhd = model_likelihoods)
plot_l_curves=function(lkhd, main=NULL, ylab=NULL, tr_nocl=NULL, stpts=NULL)
{
  if(is.null(stpts)) stpts = ncol(lkhd)
  lkhd[1,-c(1)] = NA
  ylim = range(lkhd   , na.rm=T)
  plot(lkhd[,1], typ="l", col=1, ylim=ylim, main=main, ylab=ylab)
  abline(v=tr_nocl, col="lightgrey", lty=3)
  for(stpt_ind in 1:stpts)
    lines(lkhd[,stpt_ind], col=stpt_ind)
}


## One of the MAIN SUMMARY FUNCTIONS ##
# look_at_stpts(cnvs = c(6,12,13,23,24,26,27,29, 47,75,88), real_data = F, det_ftag="04wknd_", png=T)
look_at_stpts=function(cnvs=1:1, real_data=F, seed=1, run=1, det_ftag = "04wknd_", noco=3, no_runs=500, png=F, eps=F,
      stpts_dir="/data/garganey/cardin/postdoc/CNVs/calling/stpts/")
{
  for(cnv_index in cnvs)
  {
    nocls = get_nocls(cnv_index=cnv_index, real_data=real_data, seed=seed, run=run, 
                    det_ftag = det_ftag, noco=noco, no_runs=no_runs)
    ml    = summarise_these_model_fits(cnv_index=cnv_index, tr_nocl=nocls$tr_nocl, inf_nocl=nocls$inf_nocl,
                    seed=seed, no_runs=no_runs, stpts_dir=stpts_dir, real_data=real_data, png=png)
    if(!png) browser()
  }
}

# options(warn=2); source("cnv_caller.R"); eval_all_sims(run_check=F, inf_date = "2009-05-18")
eval_all_sims=function(cnvs=1:200, project_root = "/users/cardin/CNVs/calling/", inf_date = "2009-05-18", run_check=T)
{
  exps            = sim_exps() 
  simdat_tags     = simdat_names()$simdat_tags ; 
  simdat_dates    = simdat_names()$simdat_dates; 
  for(sim in length(simdat_tags):1)
  {
    simdat_date     = simdat_dates[sim];
    simdat_tag      = simdat_tags[ sim];
    if(simdat_tag %in% c("sim_var_beta_small_colls", "sim_var_beta_low_mafs")) noco=3 else noco=10
    tests           = c(0,noco)
  
    for(test in tests)
    {
      for(i in 1:length(exps))
      {
        read_tag        = paste(simdat_tag, "_", simdat_date, sep=""); #paste("simtest2_", simdat_date, sep="")
        write_extra     = paste(simdat_tag, "_", sep="")
        run_name        = paste(write_extra, names(exps)[i], sep=""); 
        write_tag       = paste(run_name, "_", inf_date, sep="")   # results_dir_tag will be defined in the standard way within 'sim_run_caller'
        assign(x = "project_root"         , value = project_root                 , envir = .GlobalEnv); 
        assign(x = "write_tag"            , value = write_tag                    , envir = .GlobalEnv); 
        assign(x = "results_dir_tag"      , value = paste(write_tag, "/", sep=""), envir = .GlobalEnv); 
        assign(x = ".default.coutput_root", value = paste(project_root, "caller_output/", results_dir_tag, sep=""), envir = .GlobalEnv); 
        if(run_check) {
          signal_model = "t8"; if(names(exps[i])=="normal_signal") signal_model = "normal"
         #x = system(paste("wc ", get_coutput_fns(signal_model=signal_model)$details_fn     , sep=""), intern=T)
         #x = system(paste("wc ", get_coutput_fns(signal_model=signal_model)$sorted_smry_cfn, sep=""), intern=T)
          x = system(paste("ls ", get_coutput_dirs()$plots_dir, " | wc ", sep=""), intern=T)
          y = strsplit(x, split=" ")[[1]]
          nolines = as.numeric(y[which(nchar(y)>0)][1])
          cat("  ", write_tag, ": Number of lines in smry file = ", nolines, sep="", "\n")
        } else {
          cat("Processing: ", write_tag, "\n", sep="")
          evaluate_sim_study(cnvs=cnvs, run=1, sim_date=simdat_date, simdat_tag=read_tag)
          #browser();
        }
      }
    }
  }
}

## One of the MAIN SUMMARY FUNCTIONS ##
# detach(package:cnvCall); library(cnvCall, lib.loc="~/Rpackages/"); setups(4);evaluate_sim_study(cnvs=1, run=1)
# evaluate_sim_study(cnvs=1:100, real_data=F, seed=1, run=1, noco=3, no_runs=500, stop=T, write=F, png=F)
# options(error=recover); evaluate_sim_study(cnvs=1:100, real_data=F, simseed=1, infseed=1, run=1, noco=3, no_runs=500, stop=F, write=T, png=F, eps=F, pdf=T)
# options(error=recover); evaluate_sim_study(cnvs=1:95 , real_data=F, simseed=1, infseed=2, run=1, noco=3, no_runs=500, stop=F, write=T, png=F, eps=F, pdf=T)
evaluate_sim_study=function(cnvs=1:1, real_data=F, simseed=1, infseed=1, run=1, noco=3, no_runs=500, pca=T,
                      sim_date, simdat_tag, png=F, eps=F, pdf=T, stop=F, write=T)
{
  stpt_states  = get_stpt_states(seed=infseed, cnv_name=1, cnv_index=1, no_runs=no_runs)
  noco         = stpt_states[[1,1]]$opars$noco
  nocls  = matrix(NA,       nrow=length(cnvs), ncol=2    ); colnames(nocls ) = c("truth", "inferred");
  beta1s = matrix(NA,       nrow=length(cnvs), ncol=2    ); colnames(beta1s) = c("truth", "inferred");
  chisqs =  array(NA, dim=c(nrow=length(cnvs), ncol=noco)); colnames(chisqs) = paste(rep("coll", noco), 1:noco, sep="");
  corrs  =  array(NA, dim=c(nrow=length(cnvs), ncol=noco)); colnames(corrs ) = paste(rep("coll", noco), 1:noco, sep="");
  results_dir  = get_coutput_dirs()$sim_smry_dir
 #arguments    = list("simdat_tag"=write_tag, "read_seed"=1) 
  arguments    = list("simdat_tag"=simdat_tag, "read_seed"=1) 
  simpar_names = unlist(read.table(get_input_fns(arguments=arguments, cnv_index=1)$simpars_fn, header=F, as.is=T, nrow=1, sep=" "))
  for(cnv in cnvs) {
    if(cnv%%5==0) cat(".")
    cnv_index    = which(cnvs==cnv)
    cnv_name     = cnv_index
   #det_output   = read_details(real_data=real_data, seed=infseed, run=run, arguments=arguments)
   #details      = det_output$details
   #details_hdr  = det_output$details_hdr
   #proc_details = process_details(cnv_index=cnv_index, details=details, details_hdr=details_hdr, noco=noco)
   #tr_nocl      = proc_details$cnv_det[1,"nocl"]
   #inf_nocl     = proc_details$cnv_det[2,"nocl"]
    stpt_states  = get_stpt_states(seed=infseed, cnv_name=cnv_name, cnv_index=cnv, no_runs=no_runs)
    if(length(stpt_states)>1) { #stpt_states!="stpt_states_not_written"
      no_classes_tried  = get_nocls_tried(stpt_states)
      ud_name = load(get_coutput_fns(cnv_name=cnv_name)$data_fn); ung_dat = get(ud_name); rm(ud_name);
     #ung_dat           = get_sim_ung_dat(sim_date=sim_date, seed=simseed, cnv_index=cnv, pca=pca, verbose=F)
      evidences         = apply(X=stpt_states, MARGIN=c(1,2), FUN=calc_evidence, separate_collections=T, set_point = "hier_mchoice", type="bic") 
      bf_start          = stpt_states[which(evidences==max(evidences, na.rm=T), arr.ind=T)][[1]]
    } 
    fit_fn          = get_coutput_dirs()$fit_fn 
    fit_obj_name    = load(get_coutput_fns(cnv_name=cnv_name)$fit_fn)
    inf_smry        = get(fit_obj_name); rm(fit_obj_name)
    inf_args        = read.table(get_coutput_fns()$args_input_fn, as.is=T, header=F)
    best_fit        = make_objects_from_cfitoutput(inf_smry=inf_smry, inf_args=inf_args); best_fit$flags$cnv=cnv;
    best_fit$flags$DEBUG = 2

    merged_fit   = post_bic_simple_merge(fit=best_fit, pars=best_fit$opars, mode_counter="simple_nc") 
    noco         = best_fit$opars$noco
    simpars_fn   = get_input_fns(arguments=arguments, cnv_index=NULL)$simpars_fn
    simpar_dat   = unlist(read.table(simpars_fn, header=F, as.is=T, skip=cnv_index, nrow=1, sep=" "))
    simpars      = parse_sim_pars(data=simpar_dat, names=simpar_names)
    tr_nocl      = 2*as.numeric(simpars["no_alleles"])-1
    inf_nocl     = merged_fit$opars$nocl
    nocls[ cnv_index,] = c(tr_nocl, inf_nocl)
    beta1s[cnv_index,] = c(as.numeric(simpars["betas2"]), merged_fit$opars$betas[2])

    #bfclm2 = make_clm(smcs=best_fit$opars$smc, clds=best_fit$opars$cld, nocl=3, noco=3)
    chisqs[cnv_index,] = calc_chisq(sim_date=sim_date, infseed=infseed, no_runs=no_runs, cnv_index=cnv, best_fit=best_fit, ung_dat=ung_dat)
   #data_array   = get_sim_data_array(sim_date=sim_date, seed=simseed, cnv_index=cnv, pca=pca)
    simdat       = get_sim_dat(cnv_index=cnv, simdat_tag=simdat_tag, arguments=arguments)
    data_array   = simdat$data_array
    cnv_types    = simdat$cnv_types
    h_calls      = check_calls(ob = best_fit, dat=best_fit$dat, cnv_types=cnv_types, data_array=data_array, ung_dat=ung_dat) 
    max_nocl     = max(range(cnv_types)[2], range(h_calls)[2])
    corrs[cnv_index,] = calls_cor(h_calls=h_calls, cnv_types=cnv_types, nocl=max_nocl)
    if(stop) browser();
  }
  cat("Done ", write_tag, "\n", sep="")
  cc = best_fit$assoc$case_colls
  if(0) {
    make_beta1_inf_hist(beta1s=beta1s, png=png, eps=eps, pdf=pdf, nocls=nocls, pdfwidth=11, pdfheight=6, infseed=infseed)
    make_results_smry_hist(results_table=chisqs, png=png, eps=eps, pdf=pdf, nocls,   xlab     = "(Observed-Expected)^2 summary of distribution fit",
             image_fn = paste(results_dir, "Histogram_of_chisq_Inference", sep=""), cc=cc, main_all = "", main_lt7="", infseed=infseed)
    make_results_smry_hist(results_table=corrs, png=png, eps=eps, pdf=pdf, nocls,   xlab     = "r^2 with `true' copy number", 
             image_fn = paste(results_dir, "Histogram_of_call_correlations_Inference", sep=""), nocl_gt1_only=T, cc=cc, main_all = "", main_lt7="",
             infseed=infseed)
  }
  agree_table    = make_nocl_table(nocls)
  if(write) {
    if(!file.exists(results_dir)) dir.create(results_dir)
    write.table(format(agree_table), file=paste(results_dir, "model_choice_summary_seed=", infseed, ".txt", sep=""), quote=F)
    write.table(format(beta1s     ), file=paste(results_dir, "beta_inference_table_seed=", infseed, ".txt", sep=""), quote=F)
    write.table(format(chisqs     ), file=paste(results_dir, "chisqs_table_seed="        , infseed, ".txt", sep=""), quote=F)
    write.table(format(corrs      ), file=paste(results_dir, "correlations_table_seed="  , infseed, ".txt", sep=""), quote=F)
  }
  # hist(chisqs[,1], n=100, col="lightgrey")
  # save(list = ls(all=TRUE), file = "tmp_simstudy_summary.RData")
  save(list = ls(all=TRUE), file = paste(results_dir, "tmp_simstudy_summary_infseed=", infseed, ".RData", sep=""))
  #browser();
}

parse_sim_pars=function(data, names=NULL)
{
  if(is.null(names)) names = sim_pars_names()
  noco = as.numeric(data[1])
  basic_names = grep("betas", names) 
  intens_pars_index = grep("intens_pars", names)
  intens_pars = names[(basic_names+1):length(names)]
  names       = names[1:basic_names]
  simpars = data
  upto_dat = 0;  upto_nam = 0; next_nam = 1; next_dat = 1
  names(simpars)[(upto_dat+1):(upto_nam+next_dat)]  = names[(upto_nam+1):(upto_nam+next_nam)]

  upto_nam = upto_nam+next_nam; upto_dat = upto_dat+next_dat;
  next_nam = 1; next_dat = noco
  names(simpars)[(upto_dat+1):(upto_nam+next_dat)]  = paste(names[upto_nam+1], 1:next_dat, sep="")

  upto_nam = upto_nam+next_nam; upto_dat = upto_dat+next_dat;  next_nam = 2; next_dat = 2
  names(simpars)[(upto_dat+1):(upto_dat+next_dat)]  = names[(upto_nam+1):(upto_nam+next_nam)]
  upto_nam = upto_nam+next_nam; upto_dat = upto_dat+next_dat; next_nam = 1; next_dat = noco
  names(simpars)[(upto_dat+1):(upto_dat+next_dat)]  = paste(names[upto_nam+1], 1:next_dat, sep="")
  upto_nam = upto_nam+next_nam; upto_dat = upto_dat+next_dat;  next_nam = 3; next_dat = 3
  names(simpars)[(upto_dat+1):(upto_dat+next_dat)]  = names[(upto_nam+1):(upto_nam+next_nam)]
  upto_nam = upto_nam+next_nam; upto_dat = upto_dat+next_dat; 
  check = (upto_nam+1)==intens_pars_index; if(!check) cat("Warning, problem parsing simpars file.\n");
  next_nam = 1; next_dat = length(intens_pars)+(2*4) #As there are 4 distributions
  names(simpars)[(upto_dat+1):(upto_dat+next_dat)]  = c(intens_pars[1:4], paste(intens_pars[5], 1:3, sep=""),
                   paste(intens_pars[6], 1:3, sep=""), paste(intens_pars[7], 1:3, sep=""), paste(intens_pars[8], 1:3, sep=""), intens_pars[9])
  upto_nam = upto_nam+next_nam; upto_dat = upto_dat+next_dat;  next_nam = 2; next_dat = 2
  names(simpars)[(upto_dat+1):(upto_dat+next_dat)]  = names[(upto_nam+1):(upto_nam+next_nam)]
  upto_nam = upto_nam+next_nam; upto_dat = upto_dat+next_dat;  next_nam = 1; next_dat = 2
  names(simpars)[(upto_dat+1):(upto_dat+next_dat)]  = paste(names[upto_nam+1], 1:next_dat, sep="")

  return(simpars)
}

get_quantiles=function(dat_list, no_qs=1000, scale=F, shift=F)
{
  if(shift==T) {cat("Warning, do you really want to centre your distirubtions before comparing them?\n"); browser();}
  noco = length(dat_list)
  qs   = as.list(rep("", noco))
  for(coll in 1:noco)
  {
    mean  = mean(dat_list[[coll]])
    sd    =   sd(dat_list[[coll]])
    if(shift) quan_dat = dat_list[[coll]]-mean     else quan_dat = dat_list[[coll]]
    if(scale) {
      quan_dat = quan_dat/sd
    }  else   {
      quan_dat = quan_dat
    }
    probs = (0:(no_qs-1))/(no_qs-1)
    qs[[coll]] = quantile(quan_dat, probs=probs)
  }
  return(qs)
}

make_fit_dat=function(fit, no_data_points=rep(1e4, 3))
{
   noco  = fit$opars$noco
   nocl  = fit$opars$nocl
   check = (length(no_data_points)==noco); if(!check) {cat("making simulated data from model fit. Unclear on no. collection.\n"); browser();}
   clm   = make_clm(smcs=fit$opars$smc, clds=fit$opars$cld, noco=noco, nocl=nocl)
   clv   = fit$opars$clv
   gnf   = fit$opars$gnfp$gnf_lambda
   mat_no_dps  = matrix(rep(no_data_points, nocl), nrow = length(gnf))
   no_cl_dps   = round(gnf*mat_no_dps)
   sim_ung_dat = as.list(rep("", noco))
  for(coll in 1:noco)
  {
    this_coll_dat = c()
    for(cl in 1:nocl)
    {
      this_coll_dat = c(this_coll_dat, rnorm(no_cl_dps[cl,coll], mean=clm[coll,cl], sd=sqrt(clv[coll,cl])))
    }
    sim_ung_dat[[coll]] = this_coll_dat
  }
  #browser();
  #hist(sim_ung_dat[[1]], n=100)
  return(sim_ung_dat)
}

# source("process_ccc+.R"); calc_chisq(sim_date="2nd_Oct", infseed=1, cnv_index=1)
calc_chisq=function(sim_date, infseed, no_runs=500, cnv_index, stpt_states_dir = "/data/garganey/cardin/postdoc/CNVs/calling/stpts/",
                use_quantiles=F, best_fit=NULL, ung_dat=NULL)
{
  ud_name = load(get_coutput_fns(cnv_name=cnv_index)$data_fn); ung_dat = get(ud_name); rm(ud_name);
  if(is.null(best_fit)) {
    stpt_states  = get_stpt_states(stpts_dir = stpt_states_dir, seed=infseed, cnv_index=cnv_index, no_runs=no_runs, real_data=F)
    noco         = stpt_states[1,1][[1]]$opars$noco
    no_classes_tried  = get_nocls_tried(stpt_states)
    model_likelihoods = make_model_likelihoods(stpts=stpt_states)
    evidences  = apply(X=stpt_states, MARGIN=c(1,2), FUN=calc_evidence, separate_collections=T, set_point = "hier_mchoice", type="bic") 
    best_fit   = stpt_states[which(evidences==max(evidences, na.rm=T), arr.ind=T)][[1]]
  }
  noco = best_fit$opars$noco
  if(use_quantiles) no_data_points = rep(3e5, noco) else no_data_points = sapply(ung_dat, length)
  fit_dat        = make_fit_dat(fit=best_fit, no_data_points=no_data_points)
  if(use_quantiles) {
    true_quantiles = get_quantiles(dat_list=ung_dat, no_qs=1e3, scale=T, shift=T)
    inf_quantiles  = get_quantiles(dat_list=fit_dat, no_qs=1e3, scale=T, shift=T)
  } else            {
    true_quantiles = get_quantiles(dat_list=ung_dat, no_qs=3e3, scale=T, shift=F)
    inf_quantiles  = get_quantiles(dat_list=fit_dat, no_qs=3e3, scale=T, shift=F)
  }
  chisq = rep(NA, noco)
  for(coll in 1:noco)
    chisq[coll] = sum((true_quantiles[[coll]]-inf_quantiles[[coll]])^2)/length(true_quantiles[[coll]])
  return(chisq)
}


# source("diagnostics.R"); agree_table    = make_nocl_table(nocls);
make_nocl_table=function(nocls)
{
  tr_nocl_vals = sort(unique(nocls[,"truth"]), decreasing=F)
  poss_nocls   = c(min(min(nocls[,"truth"],nocls[,"inferred"])), max(nocls[,"truth"],nocls[,"inferred"]))
  agree_table  = matrix(NA, nrow=length(tr_nocl_vals), ncol=2)
  colnames(agree_table) = c("correct", "total")
  rownames(agree_table) = paste(rep("nocl=", length(tr_nocl_vals)), tr_nocl_vals, sep="")
  for(nocl in tr_nocl_vals)
  {
    nocli            = which(tr_nocl_vals==nocl)
    this_nocl_subset = nocls[which(nocls[,"truth"]==nocl),,drop=F]
    agree_table[nocli, "correct"] = sum(apply(this_nocl_subset, 1, diff)==0)
    agree_table[nocli, "total"  ] = nrow(this_nocl_subset)
  }
  return(agree_table)
}


if(0) {
  look_at_stpts(no_cnvs=1:1, real_data=F, det_ftag="04wknd_")
}


get_nocls_tried=function(stpt_states)
{
  no_classes_tried = (apply(as.matrix(rownames(stpt_states)), 1, strsplit, split="="))
  no_classes_tried = as.numeric(unlist(lapply(lapply(no_classes_tried, '[[', 1), '[[', 2)))
  return(no_classes_tried)
}

# debug_model_choice(cnv_index=87, real_data=F, seed=1, run=1, det_ftag = "04wknd_", noco=3, no_runs=500)
# debug_model_choice(cnv_index=47, real_data=T, seed=3, run=1, det_ftag = NULL, noco=3, no_runs=200, display_nocl_ind=3)
# Need to produce 'ung dat' (easy if using probe-mean) as I need it to plot model fits other than the best one.
debug_model_choice=function(cnv_index=87, real_data, seed=1, run=1, det_ftag = "04wknd_", noco=3, no_runs=500, pca=F, 
                              display_nocl_ind=1, tr_nocl=-1, sim_date="2nd_Oct")
{
  stpt_states      = get_stpt_states(seed=seed, cnv_index=cnv_index, no_runs=no_runs, real_data=real_data)
  no_classes_tried = get_nocls_tried(stpt_states)
  if(!real_data) {
    tr_nocl          = get_nocls(cnv_index=cnv_index, real_data=real_data, seed=seed, run=run, 
                         det_ftag = det_ftag, noco=noco, no_runs=no_runs)$tr_nocl
    tr_nocl_ind      = which(no_classes_tried==tr_nocl)
    display_nocl_ind    = tr_nocl_ind
    if(length(display_nocl_ind)==0) cat("True number of classes was not tried (probably too many).\n")
      display_nocl_ind = nrow(stpt_states)
  }  
  evidences  = apply(X=stpt_states, MARGIN=c(1,2), FUN=calc_evidence, separate_collections=T, set_point = "hier_mchoice", type="bic") 
  best_fit   = stpt_states[which(evidences==max(evidences, na.rm=T), arr.ind=T)][[1]]
  quick_fit_summary(fit=best_fit, write=F, filetag=NULL, seed=seed, cnv_name=NULL, cnv_index=cnv_index)
  for(stpt_ind in 1:ncol(stpt_states))
  {
    fit = stpt_states[display_nocl_ind,stpt_ind][[1]]
    look_at_calls(objects=fit, new=F, new_plot=T, png=F, ung_dat, tr_nocl=no_classes_tried[display_nocl_ind],
              cnv_index=27, cnv_name=NULL)
    browser()
    if(display_nocl_ind==1) break();
  }
}


get_nocls=function(cnv_index=1, real_data=F, seed=1, run=1, det_ftag = "04wknd_", noco=3, no_runs=500)
{
  ##All of the next five lines simply find the true (simulated) number of classes!
  det_output   = read_details(real_data=real_data, det_ftag=det_ftag, seed=seed, run=run)
  details      = det_output$details
  details_hdr  = det_output$details_hdr
  proc_details = process_details(cnv_index=cnv_index, details=details, details_hdr=details_hdr, noco=noco)
  tr_nocl      = proc_details$cnv_det[1,"nocl"]
  inf_nocl     = proc_details$cnv_det[2,"nocl"]
  nocls     = list( tr_nocl ,  inf_nocl )
  names(nocls) = c("tr_nocl", "inf_nocl")
  return(nocls)
}

read_details=function(real_data=F, seed=1, run=1, arguments)
{
  details_dir   = get_coutput_dirs()$coutput_root
  details_fn    = get_coutput_fns(arguments=arguments)$details_fn

  fake_colnames = make_colnames_for_details(details_fn)
  details_hdr   = unlist(read.table(file=  details_fn, as.is=T, header=F, fill=T, nrow=1))
  details       = as.matrix(read.table(file=details_fn, as.is=T, header=F, fill=T, skip=1, col.names=fake_colnames))
  det_return    = list( details_hdr ,  details )
  names( det_return)=c("details_hdr", "details")
  return(det_return)
}

process_details=function(cnv_index, details, details_hdr, noco)
{
    det_rows = (2*(cnv_index)-1):(2*(cnv_index))
    cnv_det  = details[det_rows,]
    cnv_det  = make_det_colnames( cnv_det=cnv_det, noco=noco, details_hdr=details_hdr)
    max_nocl = max(cnv_det[,"nocl"])
    clms     = make_moms_from_det(cnv_det=cnv_det, nocl=max_nocl, noco=noco, type="clm")
    tr_clms  = clms$tr
    inf_clms = clms$inf
    clvs     = make_moms_from_det(cnv_det=cnv_det, nocl=max_nocl, noco=noco, type="clv")
    gnfs     = make_moms_from_det(cnv_det=cnv_det, nocl=max_nocl, noco=noco, type="gnf")
  return_val = list(     cnv_det ,  clms ,  clvs ,  gnfs )
  names(return_val) = c("cnv_det", "clms", "clvs", "gnfs")
  return(return_val)
}

#  compare_infseeds(done_cnvs=1:95, corr_pch="+")
compare_infseeds=function(done_cnvs=1:95, corr_pch=19)
{
  load(file="tmp_simstudy_summary_infseed=1.RData")
  polymorphic = which(apply(nocls, 1, min)>1)
  less_than_7 = which(apply(nocls, 1, min)>1 & apply(nocls, 1, max)<7)
  poly        = intersect(polymorphic, done_cnvs)
  lt7         = intersect(less_than_7, done_cnvs)
  corrs1      = corrs
  beta1s1     = beta1s
  load(file="tmp_simstudy_summary_infseed=2.RData")
  corrs2      = corrs
  beta1s2     = beta1s
  compare_seeds_corrs( corrs1 =corrs1 , corrs2 =corrs2                           , poly=poly, lt7=lt7, pch=corr_pch)
  compare_seeds_beta1s(beta1s1=beta1s1[,"inferred"], beta1s2=beta1s2[,"inferred"], poly=poly, lt7=lt7, pch="+"     )
}

compare_seeds_corrs=function(corrs1, corrs2, poly, lt7, pch=19)
{
  xlim=c(0.5, 1);   ylim=xlim
  xlab     = "Correlations from 1st Run"; ylab = "Correlations from second run";
  image_fn = "sim_calls/Comparison_of_rsq_between_runs."
  pdffn    = paste(image_fn, "pdf", sep="")
  pdf(file=pdffn, width=11, height=6)
  par(mfcol=c(1,2))
  plot(corrs1[poly], corrs2[poly], main="All polymorphic CNVs", xlab = xlab, ylab=ylab, xlim=xlim, ylim=ylim, pch=pch)
  abline(a=0, b=1, lty=2, col="lightgrey")
  plot(corrs1[lt7 ], corrs2[lt7 ], main="CNVs with less than 7 genotypes", xlab = xlab, ylab=ylab, xlim=xlim, ylim=ylim, pch=pch)
  abline(a=0, b=1, lty=2, col="lightgrey")
  dev.off()
  cat("Range of differences in correlations between runs:", range(corrs1[poly]-corrs2[poly]),"\n")
}
compare_seeds_beta1s=function(beta1s1, beta1s2, poly, lt7, pch=19)
{
  xlim=range(c(beta1s1, beta1s2)); ylim=xlim;   
  xlab     = "Beta1 Values from 1st Run"; ylab = "Beta1 Values from second run";
  image_fn = "sim_calls/Comparison_of_beta1inf_between_runs."
  pdffn    = paste(image_fn, "pdf", sep="")
  pdf(file=pdffn, width=11, height=6)
  par(mfcol=c(1,2))
  plot(beta1s1[poly], beta1s2[poly], main="All polymorphic CNVs", xlab = xlab, ylab=ylab, xlim=xlim, ylim=ylim, pch=pch)
  abline(a=0, b=1, lty=2, col="lightgrey")
  plot(beta1s1[lt7 ], beta1s2[lt7 ], main="CNVs with less than 7 genotypes", xlab = xlab, ylab=ylab, xlim=xlim, ylim=ylim, pch=pch)
  abline(a=0, b=1, lty=2, col="lightgrey")
  dev.off()
  cat("Range of differences in beta1 inference between runs:", range(beta1s1[poly]-beta1s2[poly]),"\n")
  if(is.na(range(beta1s1[poly]-beta1s2[poly])[1])) browser();
}


quick_check_bfs=function(bfs_fn = "bayes_factors_from_true_cnv_types_glm.txt", likelihoods_fn="inferred_likelihoods_from_Nov24run.txt",
               inf_summaries_fn = "summary_of_simdata_inference.RData")
{
  bfs = read.table(file=bfs_fn, as.is=T, header=T)
  load(inf_summaries_fn)
  plot(bfs[[1]], inf_summaries[[inf_names[1]]][,"assoc_evidence"], main="agreement of glm with mapper", xlab = "BFs from true cnv types + glm", ylab = "mapper bfs", xlim=range, ylim=range)
  abline(a=0, b=1, col="grey")
  browser();
}

#November 25th - Code to compare bayes factors from program with glms deviances + bic to get bayes factors (on the simulated cnv types)
# check_bfs(inf_dirs=NULL, sim_dat_dirs=NULL, real_data=F, signal_model="normal", machine="garganey", seed=1, cnvs = 1:5, data_seed=1, writeplot=F, writedat=F)
check_bfs=function(inf_dirs=NULL, sim_dat_dirs=NULL, real_data=F, signal_model="normal", machine="garganey", seed=1, cnvs = 1:200, data_seed=1, writeplot=F, writedat=F)
{
   #currently these don't match up!
   titles         = c("No assoc, No hierarchy", "No assoc, Use hierarchy", "No assoc, No hierarchy2", "No assoc, Use hierarchy2", "Beta1 = 0.2, using hierarchy", "Beta1 = 0.1, using hierarchy")
   fname_titles   = c("No_assoc, No_hierarchy", "No_assoc_Use_hierarchy", "No_assoc_No_hierarchy2", "No_assoc_Use hierarchy2", "Beta1=0.2_using_hierarchy", "Beta1=0.1_using_hierarchy")
   stpt_dirs      = get_stpt_dirs()
   sim_calls_root = "/data/garganey/cardin/postdoc/CNVs/calling/sim_calls/"
   sim_ddir_root  = "/data/garganey/cardin/postdoc/CNVs/calling/sim_dat/"
   tags           = c("5Jan_test", "8Decrun_24Novdat_usehier", "24Novrun_24Novdat_usehier", "20Novrun_20Novdat_nohier", "20Nov_usehier")
   dfts           = c("21Nov_beta=0.2", "24Nov_beta=0.1", "20_Nov_noassoc")
   dat_subdirs    = c("21Nov_beta1=0.2", "24Nov_beta=0.1", "20Nov_noassoc" )
   colnames       = c("cnv_id", "Estimated no. classes:", "assoc_evidence", "beta1", "bic_evidence", "signal_model", "no_inds")
  #source("cnv_caller.R")
   options(error=recover)

   inf_dir_tags = c("5Jan_test", "8Dec_usehier_beta=0.1", "24Nov_usehier_ASSOC", "20Nov_nohier", "20Nov_usehier", "21Nov_usehier_ASSOC", "19Nov_noassoc_NOhier", "19Nov_noassoc_usehier")
   if(is.null(inf_dirs)) inf_dirs = paste(sim_calls_root, inf_dir_tags, sep="")
   if(is.null(sim_dat_dirs)) sim_dat_dirs = paste(sim_ddir_root, c("24Nov_beta=0.1", "20Nov_noassoc"), sep="")
   

   sets = 1:1; dsets = c(1,2,2,3,3) #dset is the data_set which corresponds to the results set (set)
   inf_summaries = as.list(rep("", length=length(tags)))
   inf_names = paste("d", inf_dir_tags, sep="")
   names(inf_summaries) = inf_names[1:3]
   for(set in 1:length(sets))
   {
     smry_fn = write_tag_to_fn(tag=tags[set], real_data=real_data, signal_model=signal_model, machine=machine, seed=seed, extra = "all_cnvs_smry", verbose=T)
     smry_fn = paste(inf_dirs[set], "/all_cnvs_smry", strsplit(smry_fn, split="all_cnvs_smry")[[1]][2], sep="")
     inf_summaries[[set]] = read.table(smry_fn, as.is=T, header=F, skip=1)
     colnames(inf_summaries[[set]]) = colnames
   }
   if(writedat) save(inf_summaries, file="summary_of_simdata_inference.RData")
   cnvs = 1:nrow(inf_summaries[[set]])

   true_bfs = lapply(1:length(unique(dsets)), function(x) return(rep(NA, nrow(inf_summaries[[1]])))); names(true_bfs) = dat_subdirs
   gen_fs  = lapply(1:length(unique(dsets)), function(x) return(as.list(rep(NA, nrow(inf_summaries[[1]])))));
   cont_fs = lapply(1:length(unique(dsets)), function(x) return(as.list(rep(NA, nrow(inf_summaries[[1]])))));
   case_fs = lapply(1:length(unique(dsets)), function(x) return(as.list(rep(NA, nrow(inf_summaries[[1]])))));
   #bfs = sapply(bfs, function(x) as.numeric(x))
   for(dset in unique(dsets))
   {
     simdat_tag = dfts[dset]
     cat("Done cnvs: ")
     for(cnv_index in cnvs)
     {
       dset_fn1 = paste("/data/garganey/cardin/postdoc/CNVs/calling/sim_dat/", dat_subdirs[dset], sep="")
       dset_fn  = paste(dset_fn1, "/", dfts[dset], "_seed=", data_seed, "_sim=", cnv_index, ".RData", sep="");  
       load(dset_fn)
       cc = total_sim$sim_pars$case_colls
       cnv_types = total_sim$simdat$cnv_types
       genc_list = lapply(cnv_types, function(x) return(as.matrix(table(x))) )
       #gen_fs[[dset]][[cnv_index]] = as.matrix(data.frame(genc_list))
       #cont_counts = rowSums(gen_fs[[dset]][[cnv_index]][,which(cc==0), drop=F])
       #cont_fs[[dset]][[cnv_index]] = normalise(cont_counts, sum=1)
       #case_counts = rowSums(gen_fs[[dset]][[cnv_index]][,which(cc==1), drop=F])
       #case_fs[[dset]][[cnv_index]] = normalise(case_counts, sum=1)
       response = c(); for(coll in 1:length(cnv_types)) response = c(response, rep(cc[coll], length(cnv_types[[coll]])))
       gadd  = glm(response~unlist(cnv_types), family=binomial())
       gnull = glm(response~1, family=binomial())
       # aic = -2 * (log-likelihood)  + 2 * #linear parameters, => logl = -1/2(aic - 2*no_pars) = no_pars - aic/2 #What is no parameters!?
       # bic = likelihood - no_pars*log(no_data_points)
       loglik_add  = 2 -  gadd$aic/2 #The additive model has a mean and an additive parameter
       loglik_null = 1 - gnull$aic/2 #The null model has a mean parameter only
       true_bfs[[dset]][cnv_index]  = (loglik_add - loglik_null) - 1*log(length(unlist(cnv_types)))
       if(cnv_index%%20 == 0) cat(cnv_index, " ", sep="")
      #if(cnv_index==10 & dset==1) browser();
     }
     cat("\nDone dataset ", dset, "\n", sep="")
   }
   if(writedat) write.table(as.matrix(data.frame(true_bfs)), file="bayes_factors_from_true_cnv_types_glm.txt", quote=F, col.names=T, row.names=T)
   range = c(min(true_bfs[[1]], inf_summaries[[inf_names[1]]][,"assoc_evidence"]), max(true_bfs[[1]], inf_summaries[[inf_names[1]]][,"assoc_evidence"]))
   if(writeplot) pdf(file="Bayes_Factors_from_true_cnv_types_and_from_map_estimator.pdf")
   plot(true_bfs[[1]], inf_summaries[[inf_names[1]]][,"assoc_evidence"], main="agreement of glm with mapper", xlab = "BFs from true cnv types + glm", ylab = "mapper bfs", xlim=range, ylim=range)
   abline(a=0, b=1, col="grey")
   if(writeplot) dev.off()
   #hist(inf_summaries[[inf_names[1]]][,"assoc_evidence"], main="beta1 inference in ", xlab = "BFs from true cnv types + glm", ylab = "mapper bfs", xlim=range, ylim=range)

 
  if(0) {
   bigbf_set =  which(inf_summaries[[inf_names[1]]][,"assoc_evidence"]>35)
   no_runs = 850
   stpt_fns = paste(stpt_dirs, "24Novrun_24Novdat_usehier_seed=", seed, "_cnv=", bigbf_set, "_no_runs=", no_runs, ".RData", sep="")
   load(stpt_fns[1])
   fit = model_fits$best_fit
   nafit = fit;  nafit$opars = nafit$oinferred$noassoc_pars; #nafit stands for 'no association model fit'
   wanted_ls = c("o_intsty", "o_asolkhd", "o_prior") # , "o_liklihd"
   assoc_bf_breakdown = cbind(fit$likelihood[wanted_ls], fit$lstore$no_assoc[wanted_ls], unlist(fit$likelihood[wanted_ls])- unlist(fit$lstore$no_assoc[wanted_ls]))
   assoc_bf_breakdown = matrix(unlist(assoc_bf_breakdown), nrow=nrow(assoc_bf_breakdown))
   assoc_bf_breakdown = rbind(assoc_bf_breakdown, colSums(assoc_bf_breakdown))
   rownames(assoc_bf_breakdown) = c(wanted_ls, "Totals")
   colnames(assoc_bf_breakdown) = c("Additive Association", "No Association", "Association - No Association")
   cat("Remember - the prior probabilities aren't used for bic!\n"); print(assoc_bf_breakdown)
   if(writeplot) pdf(file="No_Association_Model_Fit.pdf")
   plotd(nafit)
   if(writeplot) dev.off()
   if(writeplot) pdf(file="Additive_Association_Model_Fit.pdf")
   plotd(fit)
   if(writeplot) dev.off()
  }
  browser();
  likelihoods = get_likelihoods_from_stpts(no_stpts=200, seed=seed, no_runs=no_runs)
  assoc_only_bfs = (likelihoods[,"add_assoc"] - likelihoods[,"null_assoc"]) - 1*log(length(unlist(cnv_types)))
  range = c(min(true_bfs[[1]], assoc_only_bfs), max(true_bfs[[1]], assoc_only_bfs))
  if(writeplot) pdf(file="Bayes_Factors_from_true_cnv_types_and_from_assoc_model_only_from_map_estiamator.pdf")
  plot(true_bfs[[1]], assoc_only_bfs, main="agreement of glm with mapper", xlab = "BFs from true cnv types + glm", ylab = "mapper bfs using only association likelihood", xlim=range, ylim=range)
  abline(a=0, b=1, col="grey")
  if(writeplot) dev.off()
  if(writedat) write.table(likelihoods, file="inferred_likelihoods_from_Nov24run.txt", quote=F, col.names=T, row.names=F)
  browser();
  #20_Nov_noassoc_seed=1_sim=1.RData
  #20_Nov_noassoc_seed=1_sim=1.RData
  #20Nov_noassoc_seed=1_sim=1.RData
  rv = list(true_bfs, inf_summaries[[inf_names[1]]], likelihoods)
  names(rv) = c("BFs_from_truth", "summary_of_inf", "assoc_likelihoods")
  return(rv)
}

get_likelihoods_from_stpts=function(no_stpts=200, seed=1, no_runs)
{
   stpt_dirs    = get_stpt_dirs()
   wanted_lkhds = c("add_total", "add_assoc", "null_total", "null_assoc")
   likelihoods  = matrix(NA, nrow=no_stpts, ncol = length(wanted_lkhds))
   colnames(likelihoods) = wanted_lkhds
   for(cnv_index in 1:no_stpts)
   {
     stpt_fns = paste(stpt_dirs, "24Novrun_24Novdat_usehier_seed=", seed, "_cnv=", cnv_index, "_no_runs=", no_runs, ".RData", sep="")
     load(stpt_fns[1])
     likelihoods[cnv_index, "add_total"] = model_fits$best_fit$likelihood$o_liklihd
     likelihoods[cnv_index, "add_assoc"] = model_fits$best_fit$likelihood$o_asolkhd
     likelihoods[cnv_index,"null_total"] = model_fits$best_fit$lstore$no_assoc$o_liklihd
     likelihoods[cnv_index,"null_assoc"] = model_fits$best_fit$lstore$no_assoc$o_asolkhd
   }
   return(likelihoods)
}

get_stpt_dirs=function()
{
   stpts_root_dir = "/data/garganey/cardin/postdoc/CNVs/calling/stpts/"
   stpt_dirs      = paste(stpts_root_dir, c("24Nov_beta=0.1/", "21Nov_beta=0.2/", "20Nov_nohier/", "20Nov_usehier/"), sep="")
   return(stpt_dirs)
}


test_freq_converters=function(objects, precision=1e-6)
{
   objects$npars$gnfp$gnf_cases    = make_case_freqs(objects=objects, control_freqs=objects$npars$gnfp$gnf_controls, new=T)
   objects$npars$gnfp$gnf_lambda   = make_pop_freqs( objects=objects, control_freqs=objects$npars$gnfp$gnf_controls, new=T) #This USES case frequencies!
   check_control_freq              = make_control_freqs(objects=objects, pop_freqs =objects$npars$gnfp$gnf_lambda  , new=T)
   check_gnfcs  = sum(abs(check_control_freq-objects$npars$gnfp$gnf_controls))<(precision)
   if(objects$flags$DEBUG>0.5 & !check_gnfcs)  {cat("In combined_max_gnf. conversion of frequencies may be wrong.\n"); browser()}
}


