
update_params=function(objects, iter=NULL, sa=NULL, forcemv = NULL, extra_args=NULL, arguments=NULL, last_iter,
                       start_hierarchy=NULL, model_choice=NULL, model=NULL, precision=1e-10, DEBUG=1)
{
  ######## Choose next move to try ##########
  objects = choose_move(objects, sa=sa, forcemv=forcemv, start_hierarchy=start_hierarchy, model=model, last_iter=last_iter)
  objects = update_hk_pars(objects, new=T)
  objects = recalc_likelihood(objects) 
  ####Updating based on acceptance/rejection
  bad     = check_clm(objects=objects, move="update_params", cur=F)
  accept  = accept_or_reject(old_lklhd= objects$likelihood$o_liklihd, new_lklhd= objects$likelihood$n_liklihd, old_pr = objects$likelihood$o_prior, new_pr = objects$likelihood$n_prior, DEBUG=objects$flags$DEBUG)
  if(DEBUG >=3) cat("Old prior on parameters = ", objects$likelihood$o_prior, ". New prior = ", objects$likelihood$n_prior, "\n");
  if(objects$flags$DEBUG > 1.5 & objects$flags$iter>30 & model=="full_model" & objects$likelihood$o_liklihd+objects$likelihood$o_prior < (objects$likelihood$n_liklihd+objects$likelihood$n_prior - 1e4)) {cat("Implausible change in posterior!!.\n"); browser();}

  if(accept) 
  {
    objects               = update_olikelihood(objects=objects, model=model)
    objects$opars         = objects$npars
    objects$accepted_last = T ; if(objects$flags$DEBUG>1.5 & bad) {cat("In update_params. Program has suggested and accepted a state with class mean outside data-range.\n"); browser();}
  } else  {
     objects               = reset_nlikelihood(objects, model)
     objects$accepted_last = F            
     objects$npars         = objects$opars 
  }
  names(objects$accepted_last) = objects$flags$mv
  return(objects)
}


####### Update the priors and likelihoods ##########
recalc_likelihood=function(objects)
{
  check_lk(l=objects$likelihood, name="recalc 1", test_inf=F, DEBUG=objects$flags$DEBUG)  #Infinities off in this function as infinities are sometimes present in moves to be REJECTED
  objects                     = update_lpdga(objects) ##Updates objects$ninferred

  ## \/ \/ **Slow** \/ \/## (In the add_log bit)
 #objects$ninferred$lpagd     = make_lpagd(ob=objects, direct_lpdga=NULL, new=T, combined=F)
  ## /\ /\ **Slow** /\ /\##
  objects$ninferred$lpagd     = NA
  objects$ninferred$pagd      = make_pagd( ob=objects, direct_lpdga=NULL, new=T, combined=F)
  if(objects$flags$DEBUG>0.5 & (min(objects$ninferred$pagd)<0 | max(objects$ninferred$pagd)>1)) {cat("In recalc_likelihood. Error, impossible pagd.\n"); browser();}

  objects                     = calc_phen_g_gen(objects, hard_calls=!(objects$flags$sum_calls)) # Updates: objects$assoc$nlkld (association part of the model for new pars)

  ## \/ \/ **Slow** \/ \/##
  objects$likelihood$n_intsty = calc_intens_lkld(dat = objects$dat, lpdga = NULL, log=T, best_call=!(objects$flags$sum_calls), 
                                  test=test, objects=objects, new=T)  #Getting the intensity likelihood (product_(inds) sum(classes)).
  ## /\ /\ **Slow** /\ /\##

  objects$likelihood$n_asolkhd = objects$assoc$nlkld
  objects$likelihood$n_liklihd = objects$likelihood$n_intsty + objects$likelihood$n_asolkhd  #Updating the overall likelihood (intensity + association)

  #Update the prior value of the proposed parameters
  if(objects$flags$use_priors) {
    objects$likelihood$n_prior_brkdwn = calc_newprior(npars = objects$npars, npriors = objects$npriors, log=T, flags = objects$flags);
  } else {
    ## This allows Maximum Likelihood Calculations instead of maximum posterior calculations
    objects$likelihood$n_prior_brkdwn = prior_ret_val(real_priors=F) #This returns a list of 0s corresponding to log prior proabilities of 1 (so no prior affect)
  }
  objects$likelihood$n_prior        = objects$likelihood$n_prior_brkdwn$ltotal_prior
  objects$likelihood$nlposterior    = objects$likelihood$n_liklihd + objects$likelihood$n_prior #Updating the overall (log) posterior (for new pars)
  check_lk(l=objects$likelihood, name="recalc 2", test_inf=F, DEBUG=objects$flags$DEBUG)
  return(objects)
}

update_olikelihood=function(objects, model)
{
    check_lk(l=objects$likelihood, name="update ol", DEBUG=objects$flags$DEBUG)
    objects$likelihood$o_intsty       = objects$likelihood$n_intsty       # The core part of the model - the probability of the intensities given means/vars/gnfs (lambdas)
    objects$likelihood$o_asolkhd      = objects$likelihood$n_asolkhd      # Association likelihood
    objects$likelihood$o_liklihd      = objects$likelihood$n_liklihd      # Intensity + Association likelihood
    objects$likelihood$o_prior        = objects$likelihood$n_prior        # product over all parts of prior
    objects$likelihood$lposterior     = objects$likelihood$nlposterior;   # Full posterior (intensity + association + all_priors)
    objects$likelihood$o_prior_brkdwn = objects$likelihood$n_prior_brkdwn # All Priors (intensity and association)
    #Miscellaneous parts of the posterior - legacy code really - should be purged eventually.
    objects$assoc$olkld             = objects$assoc$nlkld
    objects$npriors$o_prior         = objects$likelihood$n_prior;
    if((model=="full_model" | model=="full_no_hierarchy" | model=="no_assoc")) {
      objects$likelihood$bic_evidence = "not_implemented_for_hierarchy"
    } else {
      #(!) This really doesn't need to be run every iteration, and with hier_mchoice it takes a while!
      objects$likelihood$bic_evidence   = calc_evidence(fit=objects, separate_collections=(model=="hier_nocl_choice"), set_point = NULL, type="bic", new=F) #Use current likelihood
      objects$likelihood$aic_evidence   = calc_evidence(fit=objects, separate_collections=(model=="hier_nocl_choice"), set_point = NULL, type="aic", new=F) #Use current likelihood
    }
    #Updating parameters of the fit to the new accepted values.
    objects$opriors                 = objects$npriors;
    objects$oinferred               = objects$ninferred
  return(objects)
}

reset_nlikelihood=function(objects, model=NA)
{
  check_lk(l=objects$likelihood, name="reset nl1", test_inf=F, DEBUG=objects$flags$DEBUG)
     objects$npriors                   = objects$opriors;
     objects$ninferred                 = objects$oinferred
     objects$assoc$nlkld               = objects$assoc$olkld
     objects$likelihood$n_intsty       = objects$likelihood$o_intsty
     objects$likelihood$n_asolkhd      = objects$likelihood$o_asolkhd
     objects$likelihood$n_prior_brkdwn = objects$likelihood$o_prior_brkdwn
     objects$likelihood$n_prior        = objects$likelihood$o_prior      # Product over all Priors (intensity and association)
     objects$likelihood$nlposterior    = objects$likelihood$lposterior;   # Full posterior (intensity + association + all_priors)
  check_lk(l=objects$likelihood, name="reset nl2", test_inf=T, DEBUG=objects$flags$DEBUG)
  return(objects)
}

# check_lk(l=objects$likelihood, name="recalc 1")
check_lk=function(l, name, iter=1, test_inf=T, DEBUG)
{
    if(iter==1) lik = unlist(l[-c(2,5:9)]) else lik = unlist(l[-c(  7:9)])# Some (removed) entries are NA/Inf for good reasons
    check_inf = which(abs(lik) ==Inf)>0; 
    check_na  = which(is.na(lik)    )>0;
    if(DEBUG>0.1 & length(check_na )>0           ) {cat("Warning - NA in posterior ("      , name, ": ", names(check_na) , ")\n", sep=""); browser();}
    if(DEBUG>=1  & length(check_inf)>0 & test_inf) {cat("Warning - Infinity in posterior (", name, ": ", names(check_inf), ")\n", sep=""); browser();}
}




#maximisation - remember not to use the proposal when maximising!
accept_or_reject=function(new_lklhd=NULL, old_lklhd=NULL, old_pr = NULL, new_pr = NULL, maximise=T, sa=(1-maximise), heated=F, precision=1E-10, DEBUG)
{
  if(!sa) {
    if(is.null(new_lklhd) || is.null(old_lklhd)) {cat("   Input likelihoods into accept move decider!\n"); browser(); return(1);}
    if(DEBUG>2.1 & abs((new_lklhd + new_pr) - (old_lklhd + old_pr))<precision) {cat("No difference in posterior!\n"); if(DEBUG>3) browser()}
    if(maximise) accept = (new_lklhd + new_pr) > (old_lklhd + old_pr)
    return(accept)
  } else  {
    if(is.null(new_lklhd) || is.null(old_lklhd)) {cat("   Input likelihoods into accept move decider!\n"); browser(); return(1);}
    #rand        = runif(1, 0, 1, log=T)
    rand        = log(runif(1, 0, 1))
    like_shift  = ((new_lklhd + new_pr) - (old_lklhd + old_pr))
    if(heated) {
      cooling_rate = (objects$flags$simple_model_iters/10)
      heat         = objects$flags$iter/cooling_rate
      if((like_shift*heat) > rand | heat > rand) accept=T else accept=F
    } else     {
      if(like_shift > rand) accept=T              else accept=F
    }
    if(accept & like_shift<-300 & objects$flags$iter>200) {cat("Warning! Huge drop in likelihood accepted!\n."); browser();}
    return(accept)
  }
}

#Presumably this is to do with many runs giving the same likelihood?
check_if_converged=function()
{

}



old_update_params=function(objects, iter=NULL, gridding=T, sa=NULL, forcemv = NULL, extra_args=NULL, arguments=NULL, last_iter,
                       up_to_nocl_choice=F, start_hierarchy=NULL, model_choice=NULL, model=NULL, precision=1e-10)
{
  if(                     iter==objects$flags$artificial_stop[1] & 
            objects$opars$nocl==objects$flags$artificial_stop[2] &
       objects$flags$cnv_index==objects$flags$artificial_stop[3] & 
       objects$flags$stpt_ind ==objects$flags$artificial_stop[4] ) 
  {
    cat("In 'update params'. Stopped at specified iteration.\n"); 
    debug(choose_move)
    #debug(c_clust_mean); debug(c_clust_mean_cspec);
    browser();
  }
  if(!is.null(arguments)) {
    if(iter < (arguments$set_sa_limit * max(arguments$simple_model_iters))) sa = T else sa=F;
    start_hierarchy = arguments$start_hierarchy
  } else {tmp="sa, start_hierarchy should be passed to directly update"}

  no_iters           = objects$flags$no_iters
  nocl               = objects$opars$nocl
  noco               = objects$opars$noco
  objects$flags$iter = iter
  dat2               = rowSums(objects$dat)
  ######## Choose next move to try ##########
  objects            = choose_move(objects, gridding=gridding, sa=sa, forcemv=forcemv, start_hierarchy=start_hierarchy, model=model, last_iter=last_iter)
  objects            = update_hk_pars(objects, new=T)
  mv                 = objects$flags$mv
  ######## Debugging/Checking Code ##########
  check_hkclm1 = sum(abs(objects$npars$smc - objects$npars$hkclm[,1])<precision)==noco
  if(objects$flags$DEBUG>0 & !check_hkclm1){cat("Error with means. In update pars. hkclm doesn't agreen with SMC.\n"); browser();}
  #The problem with the code below is that the 'hack' addiditions aren't known about by c_clust_mean functions so they won't avoid them!
 #check_hkclm2 = max(abs(range(objects$npars$hkclm))) < max(abs(as.numeric(rownames(objects$dat))))+precision
 #if(objects$flags$DEBUG>0 & !check_hkclm2){cat("Error with means. In update pars. hkclm goes outside data_range.\n"); browser();}
  if(is.null(dim(objects$npars$cld)) | is.null(dim(objects$npars$clv)) | is.null(dim(objects$npars$hkclm)) | is.null(dim(objects$npars$hkclv)) |
     is.null(dim(objects$opars$cld)) | is.null(dim(objects$opars$clv)) | is.null(dim(objects$opars$hkclm)) | is.null(dim(objects$opars$hkclv)) ) {
    cat("Error, last move dropped a dimension.\n"); browser()
  }
  check_inference = ((iter<=max(objects$flags$simple_model_iters)) == (model=="simple"))
  if(objects$flags$DEBUG>0.1 & !check_inference) {cat("In update_params. Unclear which model is being used (hierarchical?, association?)\n"); browser()}
  check_cols_same(objects=objects, mv=mv)
  ######## Debugging/Checking Code ##########
  if(objects$flags$DEBUG>0.1) check_clv(clv=objects$npars$clv, min_var=1E-50, objects$npars$hkclv)
  ####Calculating new likelihood
  test=!is.null(forcemv)
  recalc_likelihood(objects) 
  ####Updating based on acceptance/rejection
  acpt_smry =                c("oldl"= objects$likelihood$o_liklihd, "newl"   = objects$likelihood$n_liklihd, "old_pr"=objects$likelihood$o_prior,"new_pr"= objects$likelihood$n_prior)
  accept = accept_or_reject(old_lklhd= objects$likelihood$o_liklihd, new_lklhd= objects$likelihood$n_liklihd, old_pr = objects$likelihood$o_prior, new_pr = objects$likelihood$n_prior, DEBUG=objects$flags$DEBUG)
  ####Debugging Code
  bad    = check_clm(objects=objects, smcs=objects$npars$smc, dsts=objects$npars$cld, move="update_params", cur=T)
  if(objects$flags$DEBUG>1.5 & bad & accept) {cat("Issues with the current state (update_params[update.R]) - iteration", iter, ".\n"); browser()}
  if(objects$flags$DEBUG>2   & bad         ) {cat("Issues with the current state (update_params[update.R]) - iteration", iter, ".\n"); browser()}
  if(objects$flags$DEBUG>2.5)  cat(" old_assoc_l  = ", objects$assoc$olkld        , ", and new assoc_l = ", objects$assoc$nlkld        , "\n", sep="");
  if(objects$flags$DEBUG>2.5)  cat(" old_intsy_l  = ", objects$likelihood$o_intsty, ", and new_insty_l = ", objects$likelihood$n_intsty, "\n", sep="");
  if(objects$flags$DEBUG>2.5)  cat(" l(old_prior) = ", objects$opriors$o_prior  , ", and new prior   = ", objects$likelihood$n_prior                  , "\n", sep="");
  if(objects$flags$DEBUG>3.2) {cat(" old lowest class (col 1) = ", objects$opars$smc[1]    , ", and old class distances (col 1) = \n"); print(objects$opars$cld[1,]);cat("\n", sep=" ");}
  if(objects$flags$DEBUG>3.2) {cat(" new lowest class (col 1) = ", objects$npars$smc[1]    , ", and new class distances (col 1) = \n"); print(objects$npars$cld[1,]);cat("\n", sep=" ");}
  if(0 & (mv=="maxL_vars" | mv=="maxL_means"|1)) {
    par(mfcol=c(1,1))
    xt = as.numeric(objects$oinferred$lpdga)
    yt = as.numeric(objects$ninferred$lpdga)
    ylim = range(c(xt, yt))
    plot(xt)
    lines(yt, col="green")
  }
  if(iter< -1 & (mv=="maxL_vars" | mv=="maxL_means")) {
   #print(sd_prop)
    print(acpt_smry)
    cat("Accept = ", accept, "\n", sep="");
    browser();
  }
  if(objects$flags$DEBUG > 0.1 & is.na(objects$likelihood$lposterior)){cat("In update. Likelihood is NA.\n"); browser();}

  if(accept) 
  {
    update_olikelihood(objects=objects, model=model)
    #Core parts of the posterior - used to accept/reject moves.
    objects$opars                   = objects$npars
    objects$accepted_last           = T
    if(objects$flags$DEBUG>2) cat("  Accepted! move =", mv, "new lposterior =", objects$likelihood$lposterior, "\n")
  } else  {
     reset_nlikelihood(objects, model)
     objects$accepted_last = F
     objects$npars                     = objects$opars 
  }
  names(objects$accepted_last) = mv
  if(objects$flags$DEBUG>2.5) cat("\n");
  return(objects)
}



############################Update the Parameters#############################

