analyse_with_mclust=function(data_array, plot=T, make_pngs=F, cnv_name, nocl_range=1:9)
{
  library(mclust)
  ##USING mclust to decide the number of classes
  noco = ncol(data_array)
  md = as.list(rep("", noco))
  for(coll in 1:noco)
    md[[coll]] = apply(data_array[1,coll][[1]], 1, mean)
  data = as.vector(unlist(md))
  
  mclust_bic = mclustBIC(data=data, G=nocl_range, modelNames=NULL, prior=NULL, control=emControl(),
                 initialization=list(hcPairs=NULL, subset=NULL, noise=NULL),
                 Vinv=NULL, warn=FALSE, x=NULL)
  free_var_favoured_no_classes = names(mclust_bic[,"V"])[which.max(mclust_bic[,"V"])]
  if(plot) {
    if(make_pngs) 
      png(file=paste("mclust_calling/mclust_", cnv_name, ".png", sep=""), width=1000, height=450)
    par(mfcol=c(1,2))
    hist(data, n=100)
    main = paste("Best no classes:", free_var_favoured_no_classes)
    plot(mclust_bic)
    title(main=main)
    if(make_pngs) 
      dev.off()
  }
  cat("Favoured number of classes from mclust is:", free_var_favoured_no_classes, "\n");
  return(mclust_bic)
}


##Seeing what happens when I try to use mclust on data that fails using my code and BIC.
#Vague first exploration:
if(0)
{ 
  library(mclust)
  source("function_lib.R")
  fn = "extras/3_cl_dat_getwrong.txt"
  dat = read.table(fn, as.is=T, header=T)
  cnv_types = read.table(file="extras/3_cl_dat_getwrong_TRUE_TYPES.txt", as.is=T, header=T)
  cnv_types = as.matrix(cnv_types)
  #REMOVING THE COLLECTION dimension, hopefully consistent with the way this was done with the data!	
  cnv_types = as.vector(cnv_types) 
  dat = as.matrix(dat)
  
  #1-d version of data
  dat2 = as.vector(dat)
  nocl   = 3
  no_ind = length(dat2)
  pr_asgn = matrix(1/3, nrow = no_ind, ncol=nocl)
  
  ##Note: .Machine$double.eps ~ 2e-16
  con = emControl(eps=.Machine$double.eps, tol=c(1.e-5,sqrt(.Machine$double.eps)), itmax=c(Inf,Inf), equalPro=FALSE)
  mclust1 = meE(data = dat2, z=pr_asgn, warn=T, emControl = con)
  
  pro  = rep(1, nocl)/nocl
  #pro  = rep(1, (nocl+1))/(nocl+1)
  rdat = quantile(dat2, probs=c(0.02, 0.98))
  
  regions    = ((0:(nocl+1))/(nocl+1))*(rdat[2]-rdat[1])+rdat[1]
  mean       = regions[-c(1, length(regions))]
  sd_clust   = rep((rdat[2]-rdat[1])/(2*nocl), nocl)
  sigmasq    = (sd_clust^2)[1]
  var        = list("d" = 1, "G" = nocl, "sigmasq" = sigmasq)
  parameters = list(pro=pro, mean=mean, variance = var, VinV=NULL)
  mclust2    = emE(data = dat2, parameters=parameters, warn=T)
  
  cnv_calls  = mclust2$z
  hard_calls = apply(cnv_calls, 1, which.max)
  
  look_at_mclust_calls(dat=dat2, main="Model fit from mclust", mclust=mclust2, cnv_types=cnv_types)
  
  
  ##USING mclust to decide the number of classes
  mclust_bic = mclustBIC(data=dat, G=2:6, modelNames=NULL, prior=NULL, control=emControl(),
                 initialization=list(hcPairs=NULL, subset=NULL, noise=NULL),
                 Vinv=NULL, warn=FALSE, x=NULL)
  #mclust_bic = mclustBIC(data=dat2, G=2:6, modelNames=c("E", "V"), prior=NULL, control=emControl(),
  #               initialization=list(hcPairs=NULL, subset=NULL, noise=NULL),
  #               Vinv=NULL, warn=FALSE, x=NULL)
  plot(mclust_bic)
  
  #ylim = range(mclust_bic)
  #reduced_range = c(ylim[1], 42800)
  #plot(mclust_bic, ylim=reduced_range)
  #plot(-mclust_bic, ylim=-reduced_range[2:1])
  #dev.copy2eps(file="mclust_bic_output.eps")
  
  #dev.copy2eps(file="mclust_3cluster_output.eps")
}
  
