\name{simple_call}
\Rdversion{1.1}
\alias{simple_call}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
  Function to interactively decide number of clusters and assign cluster membership.
}
\description{
  Given input (required: data_matrix, data_array or data_and_details, see below), the function 
  returns a list of inferences made about the data and (by default) gives a plot showing the basic results.

  It is *highly recommended* that the user familiarises themselves with the basic part of 'arguments' as there
  is no completely natural default for these values.
}
\usage{
simple_call(data_matrix = NULL, data_array = NULL, data_and_details = NULL, simpleArguments = arguments, quick = F, vquick = F, chr = "NA", cnv_name = "myCNV", pos = "NA", call_thresh = 0.9, plotm = T, plotum = T)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data_matrix}{
  One way of specifying the input data (and probably the easiest and least powerful) is to suppply a matrix of measurements.
  It is assumed that there is one row of data for each individual and that the columns represent repeated measurements for each individual
  If no row names or column names are supplied then default names will be added.
}
  \item{data_array}{
  The program is capable of accommodating data with broad-scale differences between collections of individuals. Individuals measured in different batches should be placed in different collections.
  The way to specify these collections is to pass a 1xn list-matrix of data in the 'data_matrix' form.
}
  \item{data_and_details}{
  The most powerful way to supply data is using 'data_and_details'. For more information on creating this list look up the function "make_data_and_details"
}
  \item{simpleArguments}{
  A list of program parameters which specify (amongst many other things): How many iterations of each type to perform, how many components to explore, what signal model to use, whether to write output to file, what (And how many) start-points to try.
}
  \item{quick}{
  The program has a default number of iterations, to run the program more quickly but less reliably you can set quick=T (or specify the number of iterations manually)
}
  \item{vquick}{
  Setting vquick=T will cause a very small number of iterations to be performed and is really only useful for investigating output formats etc.
}
  \item{chr}{
  If 'chr' is specified then this will be present in the output data, otherwise the 'chr' value is NA
}
  \item{cnv_name}{
  If 'cnv_name' is specified then this will be present in the output data, otherwise the 'cnv_name' value is "myCNV"
}
  \item{pos}{
  If 'pos' is specified then this will be present in the output data, otherwise the 'pos' value is NA
}
  \item{call_thresh}{
  One of the outputs is a set of 'hard-calls' By default these are made at the 0.9 calling-threshold (calls with maximum posterior of assignment less than the threshold are reported as NA)
}
  \item{plotm}{
  Boolean - draw a plot of the merged calls?
}
  \item{plotum}{
  Boolean - draw a plot of the unmerged calls?
}
  \item{useLdf}{
  Boolean - Call twice, using output from the first calls to weight the probes?
}
}
\details{
  There are probably zillions of details - I'll add these as they come up I suppose.
}
\value{
 \item{best_fit }{This is a list - it gives all the details of the model fit that produced the highest model evidence}
 \item{stpt_states }{This is a matrix of lists - it gives all the details of all the model fits}
 \item{cnv_name }{If supplied then returned, otherwise NA}
 \item{calls_matrices }{This is a list - the nth element is a set of soft-calls for each individual in the nth collection - from the unmerged model fit}
 \item{unmerged_hcalls }{This is a list - the nth element is a vector giving the hard-call made for each individual in that collection - from the unmerged calls}
 \item{merged_fit }{This is a list - it gives all the details of the best model fit after merging, including the density of that model fit}
 \item{data_array }{This is a list - the nth element is the data in collection n. This will be identical to data_array if passed, if data_matrix passed then data_array[[1]]==data_matrix}
 \item{merged_hcalls }{This is a list - the nth element is a vector giving the hard-call made for each individual in that collection - from the merged calls}
 \item{sample_names }{This is a list - the nth element is a vector of sample names in the nth collection assumed to the be rownames of the data-matrix/matrices}
 \item{ung_dat }{This is a list - the nth element is a vector giving the data as transformed by the program (so that the bulk lies in (-0.05, 0.05)}
 \item{merged_calls }{This is a list - the nth element is a set of soft-calls for each individual in the nth collection - from the merged model fit}
 \item{merged_hcalls }{This is a list - the nth element is a vector giving the hard-call made for each individual in that collection - from the merged calls}
}
\references{
  I'll tell you about it if you email me at cardin@stats.ox.ac.uk
}
\author{
  Niall Cardin
}
\note{
  This documentation is very incomplete, initial draft.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
  analyse_data, set_arguments, make_data_and_details, call_cnvs, plotd, 
}
\examples{
  data_matrix = matrix(rnorm(1e5), ncol=10)
  data_matrix = rbind(data_matrix, matrix(rnorm(1e5, mean=3, sd=1.4), ncol=10))
  rownames(data_matrix) = paste("ind" , 1:nrow(data_matrix), sep="")
  colnames(data_matrix) = paste("prob", 1:ncol(data_matrix), sep="")

  arguments = set_arguments(write_results=F, quick=F, nocls=5:1, cnvs=1, collections="coll_1")
  arguments$hier_mchoice_iters = 1:100
  arguments$probe_summary      = "pca" # or you could use: "t8""mean"
  arguments$signal_model       = "normal" # or you could use: "t8"  

  qcallrv=simple_call(data_matrix=data_matrix, simpleArguments = arguments)

#####################################################################
## The function is currently defined as
function (data_matrix = NULL, data_array = NULL, data_and_details = NULL, 
    simpleArguments = arguments, quick = F, vquick = F, chr = "NA", 
    cnv_name = "myCNV", pos = "NA", call_thresh = 0.9, plotm = T, 
    plotum = T) 
{
    rv = analyse_data(dat_matrix = data_matrix, data_array = data_array, 
        data_and_details = data_and_details, quick = quick, vquick = vquick, 
        arguments = simpleArguments)
    noco = rv$best_fit$opars$noco
    collnames = rownames(rv$best_fit$opars$hkclm)
    unmerged_nocl = rv$best_fit$opars$nocl
    calls_matrices = get_calls_matrices(objects = rv$best_fit, 
        ung_dat = rv$ung_dat, collnames = names(data_array))
    chiamo_style = as.list(rep(NA, noco))
    merged_calls = as.list(rep(NA, noco))
    merged_hcalls = as.list(rep(NA, noco))
    unmerged_hcalls = as.list(rep(NA, noco))
    names(chiamo_style) = collnames
    names(merged_calls) = collnames
    names(merged_hcalls) = collnames
    names(unmerged_hcalls) = collnames
    for (coll in 1:noco) {
        chiamo_style[[coll]] = matrix(c(chr, cnv_name, pos, unmerged_nocl, 
            round(calls_matrices[[coll]], digits = 3)), nrow = 1)
        unmerged_hcalls[[coll]] = make_hcalls(calls_mat = t(calls_matrices[[coll]]), 
            call_thresh = call_thresh)
        merged_fit = post_bic_simple_merge(fit = rv$best_fit, 
            pars = rv_best_fit$opars, mode_counter = "simple_nc")
        merged_calls[[coll]] = merge_calls(calls = chiamo_style[[coll]], 
            relabelled_classes = merged_fit$relabelled_classes, 
            return_mat = T, ind_names = colnames(calls_matrices[[coll]]))
        merged_hcalls[[coll]] = make_hcalls(calls_mat = merged_calls[[coll]], 
            call_thresh = call_thresh)
    }
    if (plotm | plotum) 
        par(mfcol = c(plotm + plotum, 1))
    if (plotm) 
        plotd(objects = merged_fit, dens_vals = merged_fit$mdens_vals, 
            model_fits = merged_fit$merged_fits, main = "merged", 
            plot_combined = F, new_plot = F)
    if (plotum) 
        plotd(objects = rv$best_fit, main = "unmerged", plot_combined = F, 
            new_plot = F)
    return_val = list(rv$best_fit, rv$stpt_states, rv$cnv_name, 
        calls_matrices, unmerged_hcalls, merged_fit, rv$data_array, 
        rv$sample_names, rv$cnv_types, rv$ung_dat, merged_calls, 
        merged_hcalls)
    names(return_val) = c("best_fit", "stpt_states", "cnv_name", 
        "calls_matrices", "unmerged_hcalls", "merged_fit", "data_array", 
        "sample_names", "cnv_types", "ung_dat", "merged_calls", 
        "merged_hcalls")
    return(return_val)
  }
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ CNV }
\keyword{ calling }
\keyword{ interactive }
